from datetime import datetime

from core.managers.db_manager import db


class AuditRecord(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    timestamp = db.Column(db.DateTime, default=datetime.now)

    ip_address = db.Column(db.String())

    user_id = db.Column(db.String())
    user_name = db.Column(db.String())

    system_id = db.Column(db.String())
    system_name = db.Column(db.String())

    activity_type = db.Column(db.String())
    activity_resource = db.Column(db.String())
    activity_detail = db.Column(db.String())

    def __init__(
        self,
        ip_address,
        user_id,
        user_name,
        system_id,
        system_name,
        activity_type,
        activity_resource,
        activity_detail,
    ):
        self.id = None
        self.ip_address = ip_address
        self.user_id = user_id
        self.user_name = user_name
        self.system_id = system_id
        self.system_name = system_name
        self.activity_type = activity_type
        self.activity_resource = activity_resource
        self.activity_detail = activity_detail

    @classmethod
    def store(
        cls,
        ip_address,
        user_id,
        user_name,
        system_id,
        system_name,
        activity_type,
        activity_resource,
        activity_detail,
    ):
        audit_record = AuditRecord(
            ip_address,
            user_id,
            user_name,
            system_id,
            system_name,
            activity_type,
            activity_resource,
            activity_detail,
        )
        db.session.add(audit_record)
        db.session.commit()



import os
import re
import requests
from flask import request, Response
from flask_restful import Resource

from core.managers.auth_manager import no_auth


class Keycloak(Resource):
    matchers = [
        re.compile(
            r"^"
            + re.escape(str(os.environ["TARANIS_NG_KEYCLOAK_URL"]))
            + r"\/auth\/realms\/taranis_ng\/protocol\/openid-connect\/auth\?(response_type\=code)\&(client_id\=taranis_ng)\&(redirect_uri\=(https?%3[aA]\/\/[a-z0-9A-Z%\/\.\-_]*|https?%3[aA]%2[fF]%2[fF][a-z0-9A-Z%\/\.\-_]*))$"  # noqa
        ),  # noqa
        # login url
        re.compile(
            r"^"
            + re.escape(str(os.environ["TARANIS_NG_KEYCLOAK_URL"]))
            + r"\/auth\/realms\/taranis_ng\/login-actions\/authenticate(\??session_code\=[a-zA-Z0-9\-_]+)?(\&?\??execution\=[\w]{8}-[\w]{4}-[\w]{4}-[\w]{4}-[\w]{12})?(\&?\??client_id\=taranis_ng)?(\&?\??tab_id=[a-zA-Z0-9\-_]+)?$"  # noqa
        ),  # noqa
        # login submit url
        re.compile(
            r"^"
            + re.escape(str(os.environ["TARANIS_NG_KEYCLOAK_URL"]))
            + r"\/auth\/realms\/taranis_ng\/protocol\/openid-connect\/logout\?(redirect_uri\=(https?%3[aA]\/\/[a-z0-9A-Z%\/\.\-_]*|https?%3[aA]%2[fF]%2[fF][a-z0-9A-Z%\/\.\-_]*))$"  # noqa
        ),
        # logout url
        re.compile(
            r"^"
            + re.escape(str(os.environ["TARANIS_NG_KEYCLOAK_URL"]))
            + r"\/auth\/resources\/([^\.]*|[^\.]*\.[^\.]*|[^\.]*\.[^\.]*\.[^\.]*)$"
        ),
        # resources url
        re.compile(
            r"^"
            + re.escape(str(os.environ["TARANIS_NG_KEYCLOAK_URL"]))
            + r"\/auth\/realms\/taranis_ng\/login-actions\/required-action(\??session_code\=[a-zA-Z0-9\-_]+)?(\??\&?execution\=(UPDATE_PASSWORD))(\&?\??client_id\=taranis_ng)?(\&?\??tab_id=[a-zA-Z0-9\-_]+)?$"  # noqa
        ),
        # reset password url
    ]

    @no_auth
    def proxy(self):
        allowed = any(m.match(request.url) for m in self.matchers)
        if not allowed:
            return {"error": "Access forbidden"}, 403

        try:
            resp = requests.request(
                method=request.method,
                url=request.url.replace(
                    str(os.environ["TARANIS_NG_KEYCLOAK_URL"]) + "/",
                    os.getenv("TARANIS_NG_KEYCLOAK_INTERNAL_URL"),
                    1,
                ),
                headers={key: value for (key, value) in request.headers if key != "Host"},
                data=request.get_data(),
                cookies=request.cookies,
                proxies={"http": None, "https": None},
                allow_redirects=False,
            )

            excluded_headers = [
                "content-encoding",
                "content-length",
                "transfer-encoding",
                "connection",
            ]
            headers = [(name, value) for (name, value) in resp.raw.headers.items() if name.lower() not in excluded_headers]

            return Response(resp.content, resp.status_code, headers)
        except Exception:
            return {"error": "Internal server error"}, 500

    def get(self, path):
        return self.proxy()

    def post(self, path):
        return self.proxy()

    def put(self, path):
        return self.proxy()

    def delete(self, path):
        return self.proxy()


def initialize(api):
    api.add_resource(Keycloak, "/api/v1/auth/keycloak/<path:path>")



from flask import request
from flask_restful import Resource

from core.managers import auth_manager
from core.managers.auth_manager import access_key_required
from core.model import news_item, remote, report_item


class RemoteConnect(Resource):
    @access_key_required
    def get(self):
        return remote.RemoteAccess.connect(auth_manager.get_access_key())


class RemoteDisconnect(Resource):
    @access_key_required
    def get(self):
        return remote.RemoteAccess.disconnect(auth_manager.get_access_key())


class RemoteSyncNewsItems(Resource):
    @access_key_required
    def get(self):
        remote_access = remote.RemoteAccess.find_by_access_key(auth_manager.get_access_key())
        news_items, last_sync_time = news_item.NewsItemData.get_for_sync(remote_access.last_synced_news_items, remote_access.osint_sources)
        return {"last_sync_time": format(last_sync_time), "news_items": news_items}

    @access_key_required
    def put(self):
        remote_access = remote.RemoteAccess.find_by_access_key(auth_manager.get_access_key())
        remote_access.update_news_items_sync(request.json)


class RemoteSyncReportItems(Resource):
    @access_key_required
    def get(self):
        remote_access = remote.RemoteAccess.find_by_access_key(auth_manager.get_access_key())
        report_items, last_sync_time = report_item.ReportItem.get_for_sync(
            remote_access.last_synced_report_items, remote_access.report_item_types
        )
        return {"last_sync_time": format(last_sync_time), "report_items": report_items}

    @access_key_required
    def put(self):
        remote_access = remote.RemoteAccess.find_by_access_key(auth_manager.get_access_key())
        remote_access.update_report_items_sync(request.json)


def initialize(api):
    api.add_resource(RemoteConnect, "/api/v1/remote/connect")
    api.add_resource(RemoteDisconnect, "/api/v1/remote/disconnect")
    api.add_resource(RemoteSyncNewsItems, "/api/v1/remote/sync-news-items")
    api.add_resource(RemoteSyncReportItems, "/api/v1/remote/sync-report-items")



from flask_restful import Resource
from flask_jwt_extended import jwt_required
from flask import send_from_directory
from core.config import Config
from core.managers.log_manager import logger


class OpenAPI(Resource):
    @jwt_required()
    def get(self):
        try:
            return send_from_directory(Config.OpenAPI, "openapi3_0.yaml")
        except Exception:
            logger.log_debug_trace(f"ERROR: Loading openapi3_0.yaml from: {Config.OpenAPI}")
            return "OpenAPI file not found", 404


def initialize(api):
    api.add_resource(OpenAPI, "/api/v1/openapi", "/api/openapi.yaml", "/api/v1/openapi.yaml", "/api/openapi.yml", "/api/v1/openapi.yml")



from core.model.publishers_node import PublishersNode
from core.model.publisher import Publisher
from core.remote.publishers_api import PublishersApi
from core.model.publisher_preset import PublisherPreset
from core.managers.log_manager import logger

from shared.schema.publishers_node import PublishersNode as PublishersNodeSchema
from shared.schema.publisher import PublisherInput, PublisherInputSchema


def get_publishers_info(node: PublishersNodeSchema):
    try:
        publishers_info, status_code = PublishersApi(node.api_url, node.api_key).get_publishers_info()
    except ConnectionError:
        return f"Connection error: Could not reach {node.api_url}", 500
    except Exception:
        logger.log_debug_trace(f"Couldn't add Publisher Node: {node.name}")
        return f"Couldn't add Publisher node: {node.name}", 500

    if status_code != 200:
        return None, status_code

    return Publisher.create_all(publishers_info), status_code


def add_publishers_node(data):
    try:
        logger.log_info(data)
        publishers_info = data.pop("publishers_info")
        node = PublishersNodeSchema.create(data)
    except Exception as e:
        logger.log_debug_trace()
        return str(e), 500

    try:
        publishers = Publisher.create_all(publishers_info)
        PublishersNode.add_new(data, publishers)
    except Exception:
        logger.log_debug_trace(f"Couldn't add Publisher Node: {node.name}")
        return f"Couldn't add Publisher Node: {node.name}", 500

    return node.id, 200


def update_publishers_node(node_id, data):
    node = PublishersNodeSchema.create(data)
    publishers, status_code = get_publishers_info(node)

    if status_code != 200:
        return publishers, status_code

    try:
        PublishersNode.update(node_id, data, publishers)
    except Exception:
        logger.log_debug_trace(f"Couldn't add Publisher Node: {node.name}")
        return f"Couldn't add Publisher node: {node.name}", 500

    return node.id, status_code


def add_publisher_preset(data):
    PublisherPreset.add_new(data)


def publish(preset, data, message_title, message_body, recipients):
    publisher = preset.publisher
    node = publisher.node
    data_data = None
    data_mime = None
    if data is not None:
        data_data = data["data"]
        data_mime = data["mime_type"]

    input_data = PublisherInput(
        publisher.type,
        preset.parameter_values,
        data_mime,
        data_data,
        message_title,
        message_body,
        recipients,
    )
    input_schema = PublisherInputSchema()

    return PublishersApi(node.api_url, node.api_key).publish(input_schema.dump(input_data))



from flask import request
from flask_jwt_extended import jwt_required
from flask_restful import Resource

from core.managers.log_manager import logger
from core.model.news_item import NewsItemData
from core.model.product import Product
from core.model.report_item import ReportItem
from core.model.tag_cloud import TagCloud


class Dashboard(Resource):
    @jwt_required()
    def get(self):
        total_news_items = NewsItemData.count_all()
        total_products = Product.count_all()
        report_items_completed = ReportItem.count_all(True)
        report_items_in_progress = ReportItem.count_all(False)
        total_database_items = total_news_items + total_products + report_items_completed + report_items_in_progress
        latest_collected = NewsItemData.latest_collected()
        return {
            "total_news_items": total_news_items,
            "total_products": total_products,
            "report_items_completed": report_items_completed,
            "report_items_in_progress": report_items_in_progress,
            "total_database_items": total_database_items,
            "latest_collected": latest_collected,
        }


class Tagcloud(Resource):
    @jwt_required()
    def get(self):

        try:
            tag_cloud_day = 0
            if "tag_cloud_day" in request.args and request.args["tag_cloud_day"]:
                tag_cloud_day = min(int(request.args["tag_cloud_days"]), 7)
        except Exception as ex:
            logger.log_debug(ex)
            return "", 400

        return TagCloud.get_grouped_words(tag_cloud_day)


def initialize(api):
    api.add_resource(Dashboard, "/api/v1/dashboard-data")
    api.add_resource(Tagcloud, "/api/v1/tagcloud")



import schedule
import time
import threading


def schedule_job_every_day(interval, job_func, *args, **kwargs):
    schedule.every().day.at(interval).do(job_func, *args, **kwargs)


def schedule_job_minutes(interval, job_func, *args, **kwargs):
    schedule.every(interval).minutes.do(job_func, *args, **kwargs)


def schedule_job_on_monday(interval, job_func, *args, **kwargs):
    schedule.every().monday.at(interval).do(job_func, *args, **kwargs)


def schedule_job_on_tuesday(interval, job_func, *args, **kwargs):
    schedule.every().tuesday.at(interval).do(job_func, *args, **kwargs)


def schedule_job_on_wednesday(interval, job_func, *args, **kwargs):
    schedule.every().wednesday.at(interval).do(job_func, *args, **kwargs)


def schedule_job_on_thursday(interval, job_func, *args, **kwargs):
    schedule.every().thursday.at(interval).do(job_func, *args, **kwargs)


def schedule_job_on_friday(interval, job_func, *args, **kwargs):
    schedule.every().friday.at(interval).do(job_func, *args, **kwargs)


def schedule_job_on_saturday(interval, job_func, *args, **kwargs):
    schedule.every().saturday.at(interval).do(job_func, *args, **kwargs)


def schedule_job_on_sunday(interval, job_func, *args, **kwargs):
    schedule.every().sunday.at(interval).do(job_func, *args, **kwargs)


def run_scheduler():
    scheduler_event = threading.Event()

    class ScheduleThread(threading.Thread):
        @classmethod
        def run(cls):
            while not scheduler_event.is_set():
                schedule.run_pending()
                time.sleep(1)

    scheduler_thread = ScheduleThread()
    scheduler_thread.start()

    return scheduler_event



from core.managers.log_manager import logger
from core.auth.base_authenticator import BaseAuthenticator
from werkzeug.security import check_password_hash
from core.model.user import User


class DatabaseAuthenticator(BaseAuthenticator):
    def get_authenticator_name(self):
        return "DatabaseAuthenticator"

    def get_database_uri(self):
        return "DatabaseAuthenticator"

    def __str__(self):
        return f"Authenticator: {self.get_authenticator_name()} DB: {self.get_database_uri()}"

    def get_required_credentials(self):
        return ["username", "password"]

    def authenticate(self, credentials):
        if credentials is None:
            return BaseAuthenticator.generate_error()
        if "username" not in credentials or "password" not in credentials:
            return BaseAuthenticator.generate_error()

        user = User.find(credentials["username"])

        if check_password_hash(user.password, credentials["password"]):
            return BaseAuthenticator.generate_jwt(credentials["username"])

        logger.store_auth_error_activity(f"Authentication failed with credentials: {str(credentials)}")
        return BaseAuthenticator.generate_error()



import sqlalchemy
from marshmallow import post_load, fields
from sqlalchemy import orm, func, or_, and_
from sqlalchemy.sql.expression import cast

from core.managers.db_manager import db
from core.model.acl_entry import ACLEntry
from shared.schema.acl_entry import ItemType
from shared.schema.word_list import (
    WordListCategorySchema,
    WordListEntrySchema,
    WordListSchema,
    WordListPresentationSchema,
)


class NewWordListEntrySchema(WordListEntrySchema):
    @post_load
    def make(self, data, **kwargs):
        return WordListEntry(**data)


class NewWordListCategorySchema(WordListCategorySchema):
    entries = fields.Nested(NewWordListEntrySchema, many=True)

    @post_load
    def make(self, data, **kwargs):
        return WordListCategory(**data)


class NewWordListSchema(WordListSchema):
    categories = fields.Nested(NewWordListCategorySchema, many=True)

    @post_load
    def make(self, data, **kwargs):
        return WordList(**data)


class WordList(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(), nullable=False)
    description = db.Column(db.String(), nullable=False)
    use_for_stop_words = db.Column(db.Boolean, default=False)

    categories = db.relationship("WordListCategory", cascade="all, delete-orphan")

    def __init__(self, id, name, description, categories, use_for_stop_words):
        self.id = None
        self.name = name
        self.description = "" if description is None else description
        self.categories = categories
        self.use_for_stop_words = use_for_stop_words
        self.title = ""
        self.subtitle = ""
        self.tag = ""

    @orm.reconstructor
    def reconstruct(self):
        self.title = self.name
        self.subtitle = self.description
        self.tag = "mdi-format-list-bulleted-square"

        self.categories.sort(key=WordListCategory.sort)

    @classmethod
    def find(cls, id):
        return cls.query.get(id)

    @classmethod
    def find_by_name(cls, name):
        return cls.query.filter_by(name=name).first()

    @classmethod
    def allowed_with_acl(cls, word_list_id, user, see, access, modify):

        query = db.session.query(WordList.id).distinct().group_by(WordList.id).filter(WordList.id == word_list_id)

        query = query.outerjoin(
            ACLEntry,
            and_(
                cast(WordList.id, sqlalchemy.String) == ACLEntry.item_id,
                ACLEntry.item_type == ItemType.WORD_LIST,
            ),
        )

        query = ACLEntry.apply_query(query, user, see, access, modify)

        return query.scalar() is not None

    @classmethod
    def get_all(cls):
        return cls.query.order_by(db.asc(WordList.name)).all()

    @classmethod
    def get(cls, search, user, acl_check):
        query = cls.query.distinct().group_by(WordList.id)

        if acl_check is True:
            query = query.outerjoin(
                ACLEntry,
                and_(
                    cast(WordList.id, sqlalchemy.String) == ACLEntry.item_id,
                    ACLEntry.item_type == ItemType.WORD_LIST,
                ),
            )
            query = ACLEntry.apply_query(query, user, True, False, False)

        if search is not None:
            search_string = f"%{search.lower()}%"
            query = query.filter(
                or_(
                    func.lower(WordList.name).like(search_string),
                    func.lower(WordList.description).like(search_string),
                )
            )

        return query.order_by(db.asc(WordList.name)).all(), query.count()

    @classmethod
    def get_all_json(cls, search, user, acl_check):
        word_lists, count = cls.get(search, user, acl_check)
        schema = WordListPresentationSchema(many=True)
        return {"total_count": count, "items": schema.dump(word_lists)}

    @classmethod
    def add_new(cls, data):
        schema = NewWordListSchema()
        word_list = schema.load(data)
        db.session.add(word_list)
        db.session.commit()

    @classmethod
    def update(cls, word_list_id, data):
        schema = NewWordListSchema()
        updated_word_list = schema.load(data)
        word_list = cls.query.get(word_list_id)
        word_list.name = updated_word_list.name
        word_list.description = updated_word_list.description
        word_list.use_for_stop_words = updated_word_list.use_for_stop_words
        word_list.categories = updated_word_list.categories
        db.session.commit()

    @classmethod
    def delete(cls, id):
        word_list = cls.query.get(id)
        db.session.delete(word_list)
        db.session.commit()

    @classmethod
    def add_word_list_category(cls, id, category):
        word_list = cls.find(id)

        category_schema = NewWordListCategorySchema()
        category = category_schema.load(category)

        word_list.categories.append(category)
        db.session.commit()


class WordListCategory(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(), nullable=False)
    description = db.Column(db.String(), nullable=False)
    link = db.Column(db.String(), nullable=True, default=None)

    word_list_id = db.Column(db.Integer, db.ForeignKey("word_list.id"))

    entries = db.relationship("WordListEntry", cascade="all, delete-orphan")

    def __init__(self, name=None, description=None, link=None, entries=None):
        self.id = None
        if name in (None, ""):
            raise Exception("Empty category name!")
        self.name = name
        self.description = description or ""
        self.link = link
        self.entries = entries

    @staticmethod
    def sort(category):
        return category.name

    @classmethod
    def find(cls, id, name):
        return cls.query.filter_by(word_list_id=id).filter_by(name=name).scalar()

    @classmethod
    def get_categories(cls, id):
        categories = cls.query.filter_by(word_list_id=id).all()
        word_list_categories_schema = NewWordListCategorySchema(many=True)
        return word_list_categories_schema.dump(categories)

    @classmethod
    def add_new(cls, data):
        schema = NewWordListCategorySchema()
        word_list_category = schema.load(data)
        db.session.add(word_list_category)
        db.session.commit()


class WordListEntry(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    value = db.Column(db.String(), nullable=False)
    description = db.Column(db.String(), nullable=False)

    word_list_category_id = db.Column(db.Integer, db.ForeignKey("word_list_category.id"))

    def __init__(self, value, description):
        self.id = None
        self.value = value
        self.description = description or ""

    @classmethod
    def identical(cls, value, word_list_category_id):
        return db.session.query(
            db.exists().where(WordListEntry.value == value).where(WordListEntry.word_list_category_id == word_list_category_id)
        ).scalar()

    @classmethod
    def delete_entries(cls, id, name):
        word_list_category = WordListCategory.find(id, name)
        cls.query.filter_by(word_list_category_id=word_list_category.id).delete()
        db.session.commit()

    @classmethod
    def update_word_list_entries(cls, id, name, entries):
        word_list_category = WordListCategory.find(id, name)

        entries_schema = NewWordListEntrySchema(many=True)
        entries = entries_schema.load(entries)

        for entry in entries:
            if not WordListEntry.identical(entry.value, word_list_category.id):
                word_list_category.entries.append(entry)
                db.session.commit()

    @classmethod
    def stopwords_subquery(cls):
        return (
            db.session.query(func.lower(WordListEntry.value))
            .distinct()
            .group_by(WordListEntry.value)
            .join(
                WordListCategory,
                WordListCategory.id == WordListEntry.word_list_category_id,
            )
            .join(WordList, WordList.id == WordListCategory.word_list_id)
            .filter(WordList.use_for_stop_words is True)
            .subquery()
        )



import uuid
from datetime import datetime

from marshmallow import post_load
from sqlalchemy import orm, or_, func

from core.managers.db_manager import db
from core.managers.log_manager import logger
from shared.schema.collectors_node import (
    CollectorsNodeSchema,
)


class NewCollectorsNodeSchema(CollectorsNodeSchema):
    @post_load
    def make_collectors_node(self, data, **kwargs):
        return CollectorsNode(
            id=data["id"],
            name=data["name"],
            description=data["description"],
            api_url=data["api_url"],
            api_key=data["api_key"],
        )


class CollectorsNode(db.Model):
    id = db.Column(db.String(), primary_key=True)
    name = db.Column(db.String(), unique=True, nullable=False)
    description = db.Column(db.String())

    api_url = db.Column(db.String(), nullable=False)
    api_key = db.Column(db.String(), nullable=False)

    created = db.Column(db.DateTime, default=datetime.now)
    last_seen = db.Column(db.DateTime, default=datetime.now)

    collectors = db.relationship("Collector", back_populates="node", cascade="all")

    def __init__(self, id, name, description, api_url, api_key):
        self.id = id if id != "" else str(uuid.uuid4())
        self.name = name
        self.description = description
        self.api_url = api_url
        self.api_key = api_key
        self.tag = ""

    @orm.reconstructor
    def reconstruct(self):
        self.tag = "mdi-server-network"

    @classmethod
    def exists_by_api_key(cls, api_key):
        return db.session.query(db.exists().where(CollectorsNode.api_key == api_key)).scalar()

    @classmethod
    def get_by_api_key(cls, api_key):
        return cls.query.filter_by(api_key=api_key).first()

    @classmethod
    def get_all(cls):
        return cls.query.order_by(db.asc(CollectorsNode.name)).all()

    @classmethod
    def get(cls, search):
        query = cls.query

        if search is not None:
            search_string = "%" + search.lower() + "%"
            query = query.filter(
                or_(
                    func.lower(CollectorsNode.name).like(search_string),
                    func.lower(CollectorsNode.description).like(search_string),
                )
            )

        return query.order_by(db.asc(CollectorsNode.name)).all(), query.count()

    @classmethod
    def get_by_id(cls, id):
        return cls.query.filter_by(id=id).first()

    @classmethod
    def get_json_by_id(cls, id):
        return CollectorsNodeSchema().dump(cls.get_by_id(id))

    def find_collector_by_type(self, collector_type):
        for collector in self.collectors:
            if collector.type == collector_type:
                return collector

        return None

    @classmethod
    def get_all_json(cls, search):
        nodes, count = cls.get(search)
        node_schema = CollectorsNodeSchema(many=True)
        items = node_schema.dump(nodes)

        for i in range(len(items)):
            # calculate collector status
            #   green (last ping in time) < 60s
            #   orange (last ping late) < 300s
            #   red (no ping in a long time) > 300s
            try:
                time_inactive = datetime.now() - max(nodes[i].created, nodes[i].last_seen)
                items[i]["status"] = "green" if time_inactive.seconds < 60 else "orange" if time_inactive.seconds < 300 else "red"
            except Exception:
                logger.log_debug_trace("Cannot update collector status.")
                # if never collected before
                items[i]["status"] = "red"

        return {"total_count": count, "items": items}

    @classmethod
    def add_new(cls, node_data, collectors):
        new_node_schema = NewCollectorsNodeSchema()
        node = new_node_schema.load(node_data)
        node.collectors = collectors
        db.session.add(node)
        db.session.commit()
        return node

    @classmethod
    def update(cls, node_id, node_data, collectors):
        new_node_schema = NewCollectorsNodeSchema()
        updated_node = new_node_schema.load(node_data)
        node = cls.query.get_by_id(node_id)
        node.name = updated_node.name
        node.description = updated_node.description
        node.api_url = updated_node.api_url
        node.api_key = updated_node.api_key
        for collector in collectors:
            found = any(collector.type == existing_collector.type for existing_collector in node.collectors)

            if not found:
                node.collectors.append(collector)

        db.session.commit()
        return node

    @classmethod
    def delete(cls, node_id):
        node = cls.query.get_by_id(node_id)
        for collector in node.collectors:
            if len(collector.sources) > 0:
                raise Exception("Collectors has mapped sources")

        db.session.delete(node)
        db.session.commit()

    def updateLastSeen(self):
        self.last_seen = datetime.now()
        db.session.add(self)
        db.session.commit()



from flask_restful import Resource
from flask import request
from flask_jwt_extended import jwt_required

from core.managers import auth_manager
from core.managers.auth_manager import auth_required, no_auth
from core.model import word_list, product_type, publisher_preset
from core.model.user import User


class UserProfile(Resource):
    @jwt_required()
    def get(self):
        user = auth_manager.get_user_from_jwt()
        return User.get_profile_json(user)

    @jwt_required()
    def put(self):
        user = auth_manager.get_user_from_jwt()
        return User.update_profile(user, request.json)


class UserWordLists(Resource):
    @auth_required("ASSESS_ACCESS")
    def get(self):
        return word_list.WordList.get_all_json(None, auth_manager.get_user_from_jwt(), True)


class UserProductTypes(Resource):
    @auth_required("PUBLISH_ACCESS")
    def get(self):
        return product_type.ProductType.get_all_json(None, auth_manager.get_user_from_jwt(), False)


class UserPublisherPresets(Resource):
    @auth_required("PUBLISH_ACCESS")
    def get(self):
        return publisher_preset.PublisherPreset.get_all_json(None)


class UserSetup(Resource):
    @no_auth
    def get(self):
        if auth_manager.current_authenticator.get_authenticator_name() != "TestAuthenticator":
            return {"External Authtenicator"}
        if User.get_all_json().total_count > 0:
            return {"User setup complete"}
        return {"Initial user setup required"}


def initialize(api):
    api.add_resource(UserProfile, "/api/v1/users/my-profile")
    api.add_resource(UserWordLists, "/api/v1/users/my-word-lists")
    api.add_resource(UserProductTypes, "/api/v1/users/my-product-types")
    api.add_resource(UserPublisherPresets, "/api/v1/users/my-publisher-presets")



import requests
import sseclient
import threading
import time
import json

from core.managers import sse_manager
from core.model.news_item import NewsItemAggregate
from core.model.remote import RemoteAccess, RemoteNode
from core.model.report_item import ReportItem
from core.remote.remote_api import RemoteApi

event_handlers = {}


class EventThread(threading.Thread):
    app = None

    def __init__(self, remote_node, event_handler):
        threading.Thread.__init__(self)
        self.remote_node = remote_node
        self.event_handler = event_handler

    def run(self):
        while not self.event_handler.is_set():
            try:
                response = requests.get(
                    self.remote_node.events_url + "?channel=remote&access_key=" + self.remote_node.access_key,
                    stream=True,
                )
                client = sseclient.SSEClient(response)
                for event in client.events():
                    data = json.loads(event.data)
                    if event.event == "remote_access_disconnect":
                        if self.remote_node.event_id in data:
                            with EventThread.app.app_context():
                                self.remote_node.disconnect()
                            return

                    elif event.event == "remote_access_news_items_updated":
                        if self.remote_node.sync_news_items and self.remote_node.osint_source_group_id is not None:
                            if self.remote_node.event_id in data:
                                data, status_code = RemoteApi(
                                    self.remote_node.remote_url,
                                    self.remote_node.access_key,
                                ).get_news_items()
                                if status_code == 200:
                                    with EventThread.app.app_context():
                                        NewsItemAggregate.add_remote_news_items(
                                            data["news_items"],
                                            self.remote_node,
                                            self.remote_node.osint_source_group_id,
                                        )

                                    RemoteApi(
                                        self.remote_node.remote_url,
                                        self.remote_node.access_key,
                                    ).confirm_news_items_sync({"last_sync_time": data["last_sync_time"]})

                                    with EventThread.app.app_context():
                                        sse_manager.news_items_updated()

                    elif event.event == "remote_access_report_items_updated":
                        if self.remote_node.sync_report_items:
                            if self.remote_node.event_id in data:
                                data, status_code = RemoteApi(
                                    self.remote_node.remote_url,
                                    self.remote_node.access_key,
                                ).get_report_items()
                                if status_code == 200:
                                    with EventThread.app.app_context():
                                        ReportItem.add_remote_report_items(data["report_items"], self.remote_node.name)

                                    RemoteApi(
                                        self.remote_node.remote_url,
                                        self.remote_node.access_key,
                                    ).confirm_report_items_sync({"last_sync_time": data["last_sync_time"]})

                                    with EventThread.app.app_context():
                                        sse_manager.report_items_updated()

            except Exception:
                time.sleep(5)


def connect_to_events(remote_node):
    event_handler = threading.Event()
    event_thread = EventThread(remote_node, event_handler)
    event_handlers[remote_node.id] = event_handler
    event_thread.start()


def disconnect_from_events(remote_node):
    if remote_node.id in event_handlers:
        event_handlers[remote_node.id].set()


def verify_access_key(access_key):
    return RemoteAccess.exists_by_access_key(access_key)


def connect_to_node(node_id):
    remote_node = RemoteNode.find(node_id)
    access_info, status_code = RemoteApi(remote_node.remote_url, remote_node.access_key).connect()
    if status_code == 200:
        remote_node.connect(access_info)
        connect_to_events(remote_node)
    else:
        remote_node.disconnect()
        disconnect_from_events(remote_node)

    return access_info, status_code


def disconnect_from_node(node_id):
    remote_node = RemoteNode.find(node_id)
    RemoteApi(remote_node.remote_url, remote_node.access_key).disconnect()


def initialize(app):
    EventThread.app = app
    remote_nodes, count = RemoteNode.get(None)
    for remote_node in remote_nodes:
        if remote_node.enabled:
            connect_to_node(remote_node.id)



import io
from flask import request, jsonify, send_file
from flask_restful import Resource

from core.managers import asset_manager, auth_manager, sse_manager
from core.managers.log_manager import logger
from core.managers.auth_manager import auth_required, ACLCheck
from core.model import attribute, report_item, report_item_type


class ReportItemTypes(Resource):
    @auth_required("ANALYZE_ACCESS")
    def get(self):
        return report_item_type.ReportItemType.get_all_json(None, auth_manager.get_user_from_jwt(), True)


class ReportItemGroups(Resource):
    @auth_required("ANALYZE_ACCESS")
    def get(self):
        return report_item.ReportItem.get_groups()


class ReportItems(Resource):
    @auth_required("ANALYZE_ACCESS")
    def get(self):

        try:
            filter_keys = ["search", "completed", "incompleted", "range", "sort"]
            filter_args = {k: v for k, v in request.args.items() if k in filter_keys}

            group = None
            if "group" in request.args and request.args["group"]:
                group = int(request.args["group"])

            offset = int(request.args.get("offset", 0))
            limit = min(int(request.args.get("limit", 50)), 200)
        except Exception as ex:
            logger.log_debug(ex)
            return "", 400

        return report_item.ReportItem.get_json(group, filter_args, offset, limit, auth_manager.get_user_from_jwt())

    @auth_required("ANALYZE_CREATE")
    def post(self):
        new_report_item, status = report_item.ReportItem.add_report_item(request.json, auth_manager.get_user_from_jwt())
        if status == 200:
            asset_manager.report_item_changed(new_report_item)
            sse_manager.remote_access_report_items_updated(new_report_item.report_item_type_id)
            sse_manager.report_items_updated()

        return new_report_item.id, status


class ReportItem(Resource):
    @auth_required("ANALYZE_ACCESS", ACLCheck.REPORT_ITEM_ACCESS)
    def get(self, report_item_id):
        return report_item.ReportItem.get_detail_json(report_item_id)

    @auth_required("ANALYZE_UPDATE", ACLCheck.REPORT_ITEM_MODIFY)
    def put(self, report_item_id):
        modified, data = report_item.ReportItem.update_report_item(report_item_id, request.json, auth_manager.get_user_from_jwt())
        if modified is True:
            updated_report_item = report_item.ReportItem.find(report_item_id)
            asset_manager.report_item_changed(updated_report_item)
            sse_manager.report_item_updated(data)
            sse_manager.remote_access_report_items_updated(updated_report_item.report_item_type_id)

        return data

    @auth_required("ANALYZE_DELETE", ACLCheck.REPORT_ITEM_MODIFY)
    def delete(self, report_item_id):
        result, code = report_item.ReportItem.delete_report_item(report_item_id)
        if code == 200:
            sse_manager.report_items_updated()

        return result, code


class ReportItemData(Resource):
    @auth_required("ANALYZE_ACCESS")
    def get(self, report_item_id):

        try:
            data = {}
            if "update" in request.args and request.args["update"]:
                data["update"] = request.args["update"]
            if "add" in request.args and request.args["add"]:
                data["add"] = request.args["add"]
            if "title" in request.args and request.args["title"]:
                data["title"] = request.args["title"]
            if "title_prefix" in request.args and request.args["title_prefix"]:
                data["title_prefix"] = request.args["title_prefix"]
            if "completed" in request.args and request.args["completed"]:
                data["completed"] = request.args["completed"]
            if "attribute_id" in request.args and request.args["attribute_id"]:
                data["attribute_id"] = request.args["attribute_id"]
            if "aggregate_ids" in request.args and request.args["aggregate_ids"]:
                data["aggregate_ids"] = request.args["aggregate_ids"].split("--")
            if "remote_report_item_ids" in request.args and request.args["remote_report_item_ids"]:
                data["remote_report_item_ids"] = request.args["remote_report_item_ids"].split("--")
        except Exception:
            logger.log_debug_trace("Error getting Report Item Data")
            return "", 400

        user = auth_manager.get_user_from_jwt()
        if auth_manager.check_acl(report_item_id, ACLCheck.REPORT_ITEM_ACCESS, user) is False:
            return "", 401

        return report_item.ReportItem.get_updated_data(report_item_id, data)


class ReportItemLocks(Resource):
    @auth_required("ANALYZE_UPDATE", ACLCheck.REPORT_ITEM_MODIFY)
    def get(self, report_item_id):
        if id in sse_manager.report_item_locks:
            return jsonify(sse_manager.report_item_locks[report_item_id])
        return "{}"


class ReportItemLock(Resource):
    @auth_required("ANALYZE_UPDATE", ACLCheck.REPORT_ITEM_MODIFY)
    def put(self, report_item_id, field_id):
        sse_manager.report_item_lock(report_item_id, field_id, auth_manager.get_user_from_jwt().id)


class ReportItemUnlock(Resource):
    @auth_required("ANALYZE_UPDATE", ACLCheck.REPORT_ITEM_MODIFY)
    def put(self, report_item_id, field_id):
        sse_manager.report_item_unlock(report_item_id, field_id, auth_manager.get_user_from_jwt().id)


class ReportItemHoldLock(Resource):
    @auth_required("ANALYZE_UPDATE", ACLCheck.REPORT_ITEM_MODIFY)
    def put(self, report_item_id, field_id):
        sse_manager.report_item_hold_lock(report_item_id, field_id, auth_manager.get_user_from_jwt().id)


class ReportItemAttributeEnums(Resource):
    @auth_required("ANALYZE_ACCESS")
    def get(self, attribute_id):
        search = request.args.get(key="search", default=None)
        offset = request.args.get(key="offset", default=0)
        limit = request.args.get(key="limit", default=10)
        return attribute.AttributeEnum.get_for_attribute_json(attribute_id, search, offset, limit)


class ReportItemAddAttachment(Resource):
    @auth_required(["ANALYZE_CREATE", "ANALYZE_UPDATE"], ACLCheck.REPORT_ITEM_MODIFY)
    def post(self, report_item_id):
        file = request.files.get("file")
        if not file:
            return {"Error reading file"}

        user = auth_manager.get_user_from_jwt()
        attribute_group_item_id = request.form["attribute_group_item_id"]
        description = request.form["description"]
        data = report_item.ReportItem.add_attachment(report_item_id, attribute_group_item_id, user, file, description)
        updated_report_item = report_item.ReportItem.find(report_item_id)
        asset_manager.report_item_changed(updated_report_item)
        sse_manager.report_item_updated(data)
        sse_manager.remote_access_report_items_updated(updated_report_item.report_item_type_id)

        return data


class ReportItemRemoveAttachment(Resource):
    @auth_required("ANALYZE_UPDATE", ACLCheck.REPORT_ITEM_MODIFY)
    def delete(self, report_item_id, attribute_id):
        user = auth_manager.get_user_from_jwt()
        data = report_item.ReportItem.remove_attachment(report_item_id, attribute_id, user)
        updated_report_item = report_item.ReportItem.find(report_item_id)
        asset_manager.report_item_changed(updated_report_item)
        sse_manager.report_item_updated(data)
        sse_manager.remote_access_report_items_updated(updated_report_item.report_item_type_id)


class ReportItemDownloadAttachment(Resource):
    def get(self, report_item_id, attribute_id):
        user = auth_manager.get_user_from_jwt()
        if user is not None:
            permissions = user.get_permissions()
            if "ANALYZE_ACCESS" in permissions:
                report_item_attribute = report_item.ReportItemAttribute.find(attribute_id)
                if (
                    report_item_attribute is not None
                    and report_item_attribute.report_item.id == report_item_id
                    and report_item.ReportItem.allowed_with_acl(
                        report_item_attribute.report_item.id,
                        user,
                        False,
                        True,
                        False,
                    )
                ):
                    logger.store_user_activity(
                        user,
                        "ANALYZE_ACCESS",
                        str({"file": report_item_attribute.value}),
                    )

                    return send_file(
                        io.BytesIO(report_item_attribute.binary_data),
                        attachment_filename=report_item_attribute.value,
                        mimetype=report_item_attribute.binary_mime_type,
                        as_attachment=True,
                    )
                else:
                    logger.store_auth_error_activity("Unauthorized access attempt to Report Item Attribute")
            else:
                logger.store_auth_error_activity("Insufficient permissions")
        else:
            logger.store_auth_error_activity("Invalid JWT")


def initialize(api):
    api.add_resource(ReportItemTypes, "/api/v1/analyze/report-item-types")
    api.add_resource(ReportItemGroups, "/api/v1/analyze/report-item-groups")
    api.add_resource(ReportItems, "/api/v1/analyze/report-items")
    api.add_resource(ReportItem, "/api/v1/analyze/report-items/<int:report_item_id>")
    api.add_resource(ReportItemData, "/api/v1/analyze/report-items/<int:report_item_id>/data")
    api.add_resource(ReportItemLocks, "/api/v1/analyze/report-items/<int:report_item_id>/field-locks")
    api.add_resource(
        ReportItemLock,
        "/api/v1/analyze/report-items/<int:report_item_id>/field-locks/<int:field_id>/lock",
    )
    api.add_resource(
        ReportItemUnlock,
        "/api/v1/analyze/report-items/<int:report_item_id>/field-locks/<int:field_id>/unlock",
    )
    api.add_resource(
        ReportItemHoldLock,
        "/api/v1/analyze/report-items/<int:report_item_id>/field-locks/<int:field_id>/hold",
    )
    api.add_resource(
        ReportItemAttributeEnums,
        "/api/v1/analyze/report-item-attributes/<int:attribute_id>/enums",
    )
    api.add_resource(
        ReportItemAddAttachment,
        "/api/v1/analyze/report-items/<int:report_item_id>/file-attributes",
    )
    api.add_resource(
        ReportItemRemoveAttachment,
        "/api/v1/analyze/report-items/<int:report_item_id>/file-attributes/<int:attribute_id>",
    )
    api.add_resource(
        ReportItemDownloadAttachment,
        "/api/v1/analyze/report-items/<int:report_item_id>/file-attributes/<int:attribute_id>/file",
    )



from marshmallow import post_load, fields
import uuid
from sqlalchemy import func, or_, orm

from core.managers.db_manager import db
from core.model.bots_node import BotsNode
from core.model.parameter_value import NewParameterValueSchema
from shared.schema.bot_preset import BotPresetSchema, BotPresetPresentationSchema


class NewBotPresetSchema(BotPresetSchema):
    parameter_values = fields.List(fields.Nested(NewParameterValueSchema))

    @post_load
    def make(self, data, **kwargs):
        return BotPreset(**data)


class BotPreset(db.Model):
    id = db.Column(db.String(64), primary_key=True)
    name = db.Column(db.String(), nullable=False)
    description = db.Column(db.String())

    bot_id = db.Column(db.String, db.ForeignKey("bot.id"))
    bot = db.relationship("Bot", back_populates="presets")

    parameter_values = db.relationship("ParameterValue", secondary="bot_preset_parameter_value", cascade="all")

    def __init__(self, id, name, description, bot_id, parameter_values):
        self.id = str(uuid.uuid4())
        self.name = name
        self.description = description
        self.bot_id = bot_id
        self.parameter_values = parameter_values
        self.title = ""
        self.subtitle = ""
        self.tag = ""

    @orm.reconstructor
    def reconstruct(self):
        self.title = self.name
        self.subtitle = self.description
        self.tag = "mdi-robot"

    @classmethod
    def find(cls, preset_id):
        preset = cls.query.get(preset_id)
        return preset

    @classmethod
    def get_all(cls):
        return cls.query.order_by(db.asc(BotPreset.name)).all()

    @classmethod
    def get(cls, search):
        query = cls.query

        if search is not None:
            search_string = "%" + search.lower() + "%"
            query = query.filter(
                or_(
                    func.lower(BotPreset.name).like(search_string),
                    func.lower(BotPreset.description).like(search_string),
                )
            )

        return query.order_by(db.asc(BotPreset.name)).all(), query.count()

    @classmethod
    def get_all_json(cls, search):
        bots, count = cls.get(search)
        bot_schema = BotPresetPresentationSchema(many=True)
        return {"total_count": count, "items": bot_schema.dump(bots)}

    @classmethod
    def get_all_for_bot_json(cls, parameters):
        node = BotsNode.get_by_api_key(parameters.api_key)
        if node is not None:
            for bot in node.bots:
                if bot.type == parameters.bot_type:
                    presets_schema = BotPresetSchema(many=True)
                    return presets_schema.dump(bot.presets)

    @classmethod
    def add_new(cls, data):
        new_preset_schema = NewBotPresetSchema()
        preset = new_preset_schema.load(data)
        db.session.add(preset)
        db.session.commit()

    @classmethod
    def delete(cls, preset_id):
        preset = cls.query.get(preset_id)
        db.session.delete(preset)
        db.session.commit()

    @classmethod
    def update(cls, preset_id, data):
        new_preset_schema = NewBotPresetSchema()
        updated_preset = new_preset_schema.load(data)
        preset = cls.query.get(preset_id)
        preset.name = updated_preset.name
        preset.description = updated_preset.description

        for value in preset.parameter_values:
            for updated_value in updated_preset.parameter_values:
                if value.parameter_id == updated_value.parameter_id:
                    value.value = updated_value.value

        db.session.commit()


class BotPresetParameterValue(db.Model):
    bot_preset_id = db.Column(db.String, db.ForeignKey("bot_preset.id"), primary_key=True)
    parameter_value_id = db.Column(db.Integer, db.ForeignKey("parameter_value.id"), primary_key=True)



from flask_sqlalchemy import SQLAlchemy
from core.managers.log_manager import logger

db = SQLAlchemy()


def pre_seed(app):
    try:
        pre_seed_permissions()
        logger.log_debug("Permissions seeded")

        pre_seed_source_groups()
        logger.log_debug("Source groups seeded")

        pre_seed_roles()
        logger.log_debug("Roles seeded")

        pre_seed_default_user()
        logger.log_debug("Default users seeded")

        pre_seed_attributes()
        logger.log_debug("Attributes seeded")

        pre_seed_report_items()
        logger.log_debug("Report items seeded")

        pre_seed_wordlists()
        logger.log_debug("Wordlists seeded")

    except Exception:
        logger.log_debug_trace("Pre Seed failed")


def pre_seed_source_groups():
    from core.model.osint_source import OSINTSourceGroup

    OSINTSourceGroup.create(group_id="default", name="Default", description="Default group for uncategorized sources", default=True)


def pre_seed_permissions():
    from core.model.permission import Permission

    Permission.add("ASSESS_ACCESS", "Assess access", "Access to Assess module")
    Permission.add("ASSESS_CREATE", "Assess create", "Create news item")
    Permission.add("ASSESS_UPDATE", "Assess update", "Update news item")
    Permission.add("ASSESS_DELETE", "Assess delete", "Delete news item")

    Permission.add("ANALYZE_ACCESS", "Analyze access", "Access to Analyze module")
    Permission.add("ANALYZE_CREATE", "Analyze create", "Create report item")
    Permission.add("ANALYZE_UPDATE", "Analyze update", "Update report item")
    Permission.add("ANALYZE_DELETE", "Analyze delete", "Delete report item")

    Permission.add("PUBLISH_ACCESS", "Publish access", "Access to publish module")
    Permission.add("PUBLISH_CREATE", "Publish create", "Create product")
    Permission.add("PUBLISH_UPDATE", "Publish update", "Update product")
    Permission.add("PUBLISH_DELETE", "Publish delete", "Delete product")
    Permission.add("PUBLISH_PRODUCT", "Publish product", "Publish product")

    Permission.add("CONFIG_ACCESS", "Configuration access", "Access to Configuration module")

    Permission.add(
        "CONFIG_ORGANIZATION_ACCESS",
        "Config organizations access",
        "Access to attributes configuration",
    )
    Permission.add(
        "CONFIG_ORGANIZATION_CREATE",
        "Config organization create",
        "Create organization configuration",
    )
    Permission.add(
        "CONFIG_ORGANIZATION_UPDATE",
        "Config organization update",
        "Update organization configuration",
    )
    Permission.add(
        "CONFIG_ORGANIZATION_DELETE",
        "Config organization delete",
        "Delete organization configuration",
    )

    Permission.add("CONFIG_USER_ACCESS", "Config users access", "Access to users configuration")
    Permission.add("CONFIG_USER_CREATE", "Config user create", "Create user configuration")
    Permission.add("CONFIG_USER_UPDATE", "Config user update", "Update user configuration")
    Permission.add("CONFIG_USER_DELETE", "Config user delete", "Delete user configuration")

    Permission.add("CONFIG_ROLE_ACCESS", "Config roles access", "Access to roles configuration")
    Permission.add("CONFIG_ROLE_CREATE", "Config role create", "Create role configuration")
    Permission.add("CONFIG_ROLE_UPDATE", "Config role update", "Update role configuration")
    Permission.add("CONFIG_ROLE_DELETE", "Config role delete", "Delete role configuration")

    Permission.add("CONFIG_ACL_ACCESS", "Config acls access", "Access to acls configuration")
    Permission.add("CONFIG_ACL_CREATE", "Config acl create", "Create acl configuration")
    Permission.add("CONFIG_ACL_UPDATE", "Config acl update", "Update acl configuration")
    Permission.add("CONFIG_ACL_DELETE", "Config acl delete", "Delete acl configuration")

    Permission.add(
        "CONFIG_PRODUCT_TYPE_ACCESS",
        "Config product types access",
        "Access to product types configuration",
    )
    Permission.add(
        "CONFIG_PRODUCT_TYPE_CREATE",
        "Config product type create",
        "Create product type configuration",
    )
    Permission.add(
        "CONFIG_PRODUCT_TYPE_UPDATE",
        "Config product type update",
        "Update product type configuration",
    )
    Permission.add(
        "CONFIG_PRODUCT_TYPE_DELETE",
        "Config product type delete",
        "Delete product type configuration",
    )

    Permission.add(
        "CONFIG_ATTRIBUTE_ACCESS",
        "Config attributes access",
        "Access to attributes configuration",
    )
    Permission.add(
        "CONFIG_ATTRIBUTE_CREATE",
        "Config attribute create",
        "Create attribute configuration",
    )
    Permission.add(
        "CONFIG_ATTRIBUTE_UPDATE",
        "Config attribute update",
        "Update attribute configuration",
    )
    Permission.add(
        "CONFIG_ATTRIBUTE_DELETE",
        "Config attribute delete",
        "Delete attribute configuration",
    )

    Permission.add(
        "CONFIG_REPORT_TYPE_ACCESS",
        "Config report item types access",
        "Access to report item types configuration",
    )
    Permission.add(
        "CONFIG_REPORT_TYPE_CREATE",
        "Config report item type create",
        "Create report item type configuration",
    )
    Permission.add(
        "CONFIG_REPORT_TYPE_UPDATE",
        "Config report item type update",
        "Update report item type configuration",
    )
    Permission.add(
        "CONFIG_REPORT_TYPE_DELETE",
        "Config report item type delete",
        "Delete report item type configuration",
    )

    Permission.add(
        "CONFIG_WORD_LIST_ACCESS",
        "Config word lists access",
        "Access to word lists configuration",
    )
    Permission.add(
        "CONFIG_WORD_LIST_CREATE",
        "Config word list create",
        "Create word list configuration",
    )
    Permission.add(
        "CONFIG_WORD_LIST_UPDATE",
        "Config word list update",
        "Update word list configuration",
    )
    Permission.add(
        "CONFIG_WORD_LIST_DELETE",
        "Config word list delete",
        "Delete word list configuration",
    )

    Permission.add(
        "CONFIG_COLLECTORS_NODE_ACCESS",
        "Config collectors nodes access",
        "Access to collectors nodes configuration",
    )
    Permission.add(
        "CONFIG_COLLECTORS_NODE_CREATE",
        "Config collectors node create",
        "Create collectors node configuration",
    )
    Permission.add(
        "CONFIG_COLLECTORS_NODE_UPDATE",
        "Config collectors node update",
        "Update collectors node configuration",
    )
    Permission.add(
        "CONFIG_COLLECTORS_NODE_DELETE",
        "Config collectors node delete",
        "Delete collectors node configuration",
    )

    Permission.add(
        "CONFIG_OSINT_SOURCE_ACCESS",
        "Config OSINT source access",
        "Access to OSINT sources configuration",
    )
    Permission.add(
        "CONFIG_OSINT_SOURCE_CREATE",
        "Config OSINT source create",
        "Create OSINT source configuration",
    )
    Permission.add(
        "CONFIG_OSINT_SOURCE_UPDATE",
        "Config OSINT source update",
        "Update OSINT source configuration",
    )
    Permission.add(
        "CONFIG_OSINT_SOURCE_DELETE",
        "Config OSINT source delete",
        "Delete OSINT source configuration",
    )

    Permission.add(
        "CONFIG_OSINT_SOURCE_GROUP_ACCESS",
        "Config OSINT source group access",
        "Access to OSINT sources groups configuration",
    )
    Permission.add(
        "CONFIG_OSINT_SOURCE_GROUP_CREATE",
        "Config OSINT source group create",
        "Create OSINT source group configuration",
    )
    Permission.add(
        "CONFIG_OSINT_SOURCE_GROUP_UPDATE",
        "Config OSINT source group update",
        "Update OSINT source group configuration",
    )
    Permission.add(
        "CONFIG_OSINT_SOURCE_GROUP_DELETE",
        "Config OSINT source group delete",
        "Delete OSINT source group configuration",
    )

    Permission.add(
        "CONFIG_REMOTE_ACCESS_ACCESS",
        "Config remote access access",
        "Access to remote access configuration",
    )
    Permission.add(
        "CONFIG_REMOTE_ACCESS_CREATE",
        "Config remote access create",
        "Create remote access configuration",
    )
    Permission.add(
        "CONFIG_REMOTE_ACCESS_UPDATE",
        "Config remote access update",
        "Update remote access configuration",
    )
    Permission.add(
        "CONFIG_REMOTE_ACCESS_DELETE",
        "Config remote access delete",
        "Delete remote access configuration",
    )

    Permission.add(
        "CONFIG_REMOTE_NODE_ACCESS",
        "Config remote nodes access",
        "Access to remote nodes configuration",
    )
    Permission.add(
        "CONFIG_REMOTE_NODE_CREATE",
        "Config remote node create",
        "Create remote node configuration",
    )
    Permission.add(
        "CONFIG_REMOTE_NODE_UPDATE",
        "Config remote node update",
        "Update remote node configuration",
    )
    Permission.add(
        "CONFIG_REMOTE_NODE_DELETE",
        "Config remote node delete",
        "Delete remote node configuration",
    )

    Permission.add(
        "CONFIG_PRESENTERS_NODE_ACCESS",
        "Config presenters nodes access",
        "Access to presenters nodes configuration",
    )
    Permission.add(
        "CONFIG_PRESENTERS_NODE_CREATE",
        "Config presenters node create",
        "Create presenters node configuration",
    )
    Permission.add(
        "CONFIG_PRESENTERS_NODE_UPDATE",
        "Config presenters node update",
        "Update presenters node configuration",
    )
    Permission.add(
        "CONFIG_PRESENTERS_NODE_DELETE",
        "Config presenters node delete",
        "Delete presenters node configuration",
    )

    Permission.add(
        "CONFIG_PUBLISHERS_NODE_ACCESS",
        "Config publishers nodes access",
        "Access to publishers nodes configuration",
    )
    Permission.add(
        "CONFIG_PUBLISHERS_NODE_CREATE",
        "Config publishers node create",
        "Create publishers node configuration",
    )
    Permission.add(
        "CONFIG_PUBLISHERS_NODE_UPDATE",
        "Config publishers node update",
        "Update publishers node configuration",
    )
    Permission.add(
        "CONFIG_PUBLISHERS_NODE_DELETE",
        "Config publishers node delete",
        "Delete publishers node configuration",
    )

    Permission.add(
        "CONFIG_PUBLISHER_PRESET_ACCESS",
        "Config publisher presets access",
        "Access to publisher presets configuration",
    )
    Permission.add(
        "CONFIG_PUBLISHER_PRESET_CREATE",
        "Config publisher preset create",
        "Create publisher preset configuration",
    )
    Permission.add(
        "CONFIG_PUBLISHER_PRESET_UPDATE",
        "Config publisher preset update",
        "Update publisher preset configuration",
    )
    Permission.add(
        "CONFIG_PUBLISHER_PRESET_DELETE",
        "Config publisher preset delete",
        "Delete publisher preset configuration",
    )

    Permission.add(
        "CONFIG_BOTS_NODE_ACCESS",
        "Config bots nodes access",
        "Access to bots nodes configuration",
    )
    Permission.add(
        "CONFIG_BOTS_NODE_CREATE",
        "Config bots node create",
        "Create bots node configuration",
    )
    Permission.add(
        "CONFIG_BOTS_NODE_UPDATE",
        "Config bots node update",
        "Update bots node configuration",
    )
    Permission.add(
        "CONFIG_BOTS_NODE_DELETE",
        "Config bots node delete",
        "Delete bots node configuration",
    )

    Permission.add(
        "CONFIG_BOT_PRESET_ACCESS",
        "Config bot presets access",
        "Access to bot presets configuration",
    )
    Permission.add(
        "CONFIG_BOT_PRESET_CREATE",
        "Config bot preset create",
        "Create bot preset configuration",
    )
    Permission.add(
        "CONFIG_BOT_PRESET_UPDATE",
        "Config bot preset update",
        "Update bot preset configuration",
    )
    Permission.add(
        "CONFIG_BOT_PRESET_DELETE",
        "Config bot preset delete",
        "Delete bot preset configuration",
    )

    Permission.add(
        "CONFIG_PUBLISHERS_NODE_ACCESS",
        "Config publishers nodes access",
        "Access to publishers nodes configuration",
    )
    Permission.add(
        "CONFIG_PUBLISHERS_NODE_CREATE",
        "Config publishers node create",
        "Create publishers node configuration",
    )
    Permission.add(
        "CONFIG_PUBLISHERS_NODE_UPDATE",
        "Config publishers node update",
        "Update publishers node configuration",
    )
    Permission.add(
        "CONFIG_PUBLISHERS_NODE_DELETE",
        "Config publishers node delete",
        "Delete publishers node configuration",
    )

    Permission.add(
        "CONFIG_PUBLISHER_PRESET_ACCESS",
        "Config publisher presets access",
        "Access to publisher presets configuration",
    )
    Permission.add(
        "CONFIG_PUBLISHER_PRESET_CREATE",
        "Config publisher preset create",
        "Create publisher preset configuration",
    )
    Permission.add(
        "CONFIG_PUBLISHER_PRESET_UPDATE",
        "Config publisher preset update",
        "Update publisher preset configuration",
    )
    Permission.add(
        "CONFIG_PUBLISHER_PRESET_DELETE",
        "Config publisher preset delete",
        "Delete publisher preset configuration",
    )

    Permission.add(
        "CONFIG_PRESENTERS_NODE_ACCESS",
        "Config presenters nodes access",
        "Access to presenters nodes configuration",
    )
    Permission.add(
        "CONFIG_PRESENTERS_NODE_CREATE",
        "Config presenters node create",
        "Create presenters node configuration",
    )
    Permission.add(
        "CONFIG_PRESENTERS_NODE_UPDATE",
        "Config presenters node update",
        "Update presenters node configuration",
    )
    Permission.add(
        "CONFIG_PRESENTERS_NODE_DELETE",
        "Config presenters node delete",
        "Delete presenters node configuration",
    )

    Permission.add("MY_ASSETS_ACCESS", "My Assets access", "Access to My Assets module")
    Permission.add(
        "MY_ASSETS_CREATE",
        "My Assets create",
        "Creation of products in My Assets module",
    )
    Permission.add(
        "MY_ASSETS_CONFIG",
        "My Assets config",
        "Configuration of access and groups in My Assets module",
    )
    Permission.add(
        "CONFIG_NODE_ACCESS",
        "Config nodes access",
        "Access to all nodes from configuration",
    )


def pre_seed_roles():
    from core.model.role import Role
    from core.model.user import User  # noqa
    from core.model.organization import Organization  # noqa
    from core.model.permission import Permission

    admin_permissions = [{"id": perm.id} for perm in Permission.get_all()]
    if not Role.find(1):
        Role.add_new(
            {
                "id": "1",
                "name": "Admin",
                "description": "Administrator role",
                "permissions": admin_permissions,
            }
        )
    if not Role.find(2):
        default_user_permissions = [
            {"id": "ASSESS_ACCESS"},
            {"id": "ASSESS_CREATE"},
            {"id": "ASSESS_UPDATE"},
            {"id": "ASSESS_DELETE"},
            {"id": "ANALYZE_ACCESS"},
            {"id": "ANALYZE_CREATE"},
            {"id": "ANALYZE_UPDATE"},
            {"id": "ANALYZE_DELETE"},
            {"id": "PUBLISH_ACCESS"},
            {"id": "PUBLISH_CREATE"},
            {"id": "PUBLISH_UPDATE"},
            {"id": "PUBLISH_DELETE"},
            {"id": "PUBLISH_PRODUCT"},
        ]
        Role.add_new(
            {
                "id": "2",
                "name": "User",
                "description": "Basic user role",
                "permissions": default_user_permissions,
            }
        )


def pre_seed_attributes():
    from core.model.attribute import Attribute

    base_attr = {
        "id": -1,
        "name": "Text",
        "description": "Simple text box",
        "type": "STRING",
        "default_value": "",
        "validator": "NONE",
        "validator_parameter": "",
        "attribute_enums": [],
        "attribute_enums_total_count": 0,
    }

    if not db.session.query(Attribute).filter_by(name="Text").first():
        attr = {
            **base_attr,
            **{"name": "Text", "description": "Simple text box", "type": "STRING"},
        }
        Attribute.add_attribute(attr)

    if not db.session.query(Attribute).filter_by(name="Text Area").first():
        attr = {
            **base_attr,
            **{"name": "Text Area", "description": "Simple text area", "type": "TEXT"},
        }
        Attribute.add_attribute(attr)

    if not db.session.query(Attribute).filter_by(name="TLP").first():
        attr = {
            **base_attr,
            **{
                "name": "TLP",
                "description": "Traffic Light Protocol element",
                "type": "TLP",
            },
        }
        Attribute.add_attribute(attr)

    if not db.session.query(Attribute).filter_by(name="CPE").first():
        attr = {
            **base_attr,
            **{
                "name": "CPE",
                "description": "Common Platform Enumeration element",
                "type": "CPE",
            },
        }
        Attribute.add_attribute(attr)

    if not db.session.query(Attribute).filter_by(name="CVSS").first():
        attr = {
            **base_attr,
            **{
                "name": "CVSS",
                "description": "Common Vulnerability Scoring System element",
                "type": "CVSS",
            },
        }
        Attribute.add_attribute(attr)

    if not db.session.query(Attribute).filter_by(name="CVE").first():
        attr = {
            **base_attr,
            **{
                "name": "CVE",
                "description": "Common Vulnerabilities and Exposures element",
                "type": "CVE",
            },
        }
        Attribute.add_attribute(attr)

    if not db.session.query(Attribute).filter_by(name="Date").first():
        attr = {
            **base_attr,
            **{"name": "Date", "description": "Date picker", "type": "DATE"},
        }
        Attribute.add_attribute(attr)

    if not db.session.query(Attribute).filter_by(name="Confidentiality").first():
        attr_enum = [
            {"index": 0, "value": "UNRESTRICTED", "description": ""},
            {"index": 1, "value": "CLASSIFIED", "description": ""},
            {"index": 2, "value": "CONFIDENTIAL", "description": ""},
            {"index": 3, "value": "SECRET", "description": ""},
            {"index": 4, "value": "TOP SECRET", "description": ""},
        ]
        attr = {
            **base_attr,
            **{
                "name": "Confidentiality",
                "description": "Radio box for confidentiality level",
                "type": "RADIO",
                "attribute_enums": attr_enum,
                "attribute_enums_total_count": len(attr_enum),
            },
        }
        Attribute.add_attribute(attr)

    if not db.session.query(Attribute).filter_by(name="Impact").first():
        attr_enum = [
            {
                "index": 0,
                "value": "Malicious code execution affecting overall confidentiality, integrity, and availability of the system",
                "description": "",
            },
            {"index": 1, "value": "Malicious code execution", "description": ""},
            {"index": 2, "value": "Denial of service", "description": ""},
            {"index": 3, "value": "Privilege escalation", "description": ""},
            {"index": 4, "value": "Information exposure", "description": ""},
            {
                "index": 5,
                "value": "Unauthorized access to the system",
                "description": "",
            },
            {"index": 6, "value": "Unauthorized change in system", "description": ""},
        ]
        attr = {
            **base_attr,
            **{
                "name": "Impact",
                "description": "Combo box for impact level",
                "type": "ENUM",
                "attribute_enums": attr_enum,
                "attribute_enums_total_count": len(attr_enum),
            },
        }
        Attribute.add_attribute(attr)

    if not db.session.query(Attribute).filter_by(name="Additional Data").first():
        attr_enum = [
            {"index": 0, "value": "For Intrusion Detection System", "description": ""},
            {"index": 1, "value": "Disable Correlation", "description": ""},
        ]
        attr = {
            **base_attr,
            **{
                "name": "Additional Data",
                "description": "Radio box for MISP additional data",
                "type": "RADIO",
                "attribute_enums": attr_enum,
                "attribute_enums_total_count": len(attr_enum),
            },
        }
        Attribute.add_attribute(attr)

    if not db.session.query(Attribute).filter_by(name="MISP Event Distribution").first():
        attr_enum = [
            {"index": 0, "value": "Your organisation only", "description": ""},
            {"index": 1, "value": "This community only", "description": ""},
            {"index": 2, "value": "Connected communities", "description": ""},
            {"index": 3, "value": "All communities", "description": ""},
        ]
        attr = {
            **base_attr,
            **{
                "name": "MISP Event Distribution",
                "description": "Combo box for MISP event distribution",
                "type": "ENUM",
                "attribute_enums": attr_enum,
                "attribute_enums_total_count": len(attr_enum),
            },
        }
        Attribute.add_attribute(attr)

    if not db.session.query(Attribute).filter_by(name="MISP Event Threat Level").first():
        attr_enum = [
            {"index": 0, "value": "High", "description": ""},
            {"index": 1, "value": "Medium", "description": ""},
            {"index": 2, "value": "Low", "description": ""},
            {"index": 3, "value": "Undefined", "description": ""},
        ]
        attr = {
            **base_attr,
            **{
                "name": "MISP Event Threat Level",
                "description": "Combo box for MISP event threat level",
                "type": "ENUM",
                "attribute_enums": attr_enum,
                "attribute_enums_total_count": len(attr_enum),
            },
        }
        Attribute.add_attribute(attr)

    if not db.session.query(Attribute).filter_by(name="MISP Event Analysis").first():
        attr_enum = [
            {"index": 0, "value": "Initial", "description": ""},
            {"index": 1, "value": "Ongoing", "description": ""},
            {"index": 2, "value": "Completed", "description": ""},
        ]
        attr = {
            **base_attr,
            **{
                "name": "MISP Event Analysis",
                "description": "Combo box for MISP event analysis",
                "type": "ENUM",
                "attribute_enums": attr_enum,
                "attribute_enums_total_count": len(attr_enum),
            },
        }
        Attribute.add_attribute(attr)

    if not db.session.query(Attribute).filter_by(name="MISP Attribute Category").first():
        attr_enum = [
            {"index": 0, "value": "Internal reference", "description": ""},
            {"index": 1, "value": "Targeting data", "description": ""},
            {"index": 2, "value": "Antivirus detection", "description": ""},
            {"index": 3, "value": "Payload delivery", "description": ""},
            {"index": 4, "value": "Artifacts dropped", "description": ""},
            {"index": 5, "value": "Payload installation", "description": ""},
            {"index": 6, "value": "Persistence mechanism", "description": ""},
            {"index": 7, "value": "Network activity", "description": ""},
            {"index": 8, "value": "Payload type", "description": ""},
            {"index": 9, "value": "Attribution", "description": ""},
            {"index": 10, "value": "External analysis", "description": ""},
            {"index": 11, "value": "Financial fraud", "description": ""},
            {"index": 12, "value": "Support Tool", "description": ""},
            {"index": 13, "value": "Social network", "description": ""},
            {"index": 14, "value": "Person", "description": ""},
            {"index": 15, "value": "Other", "description": ""},
        ]
        attr = {
            **base_attr,
            **{
                "name": "MISP Attribute Category",
                "description": "Combo box for MISP attribute category",
                "type": "ENUM",
                "attribute_enums": attr_enum,
                "attribute_enums_total_count": len(attr_enum),
            },
        }
        Attribute.add_attribute(attr)

    if not db.session.query(Attribute).filter_by(name="MISP Attribute Type").first():
        attr_enum = [
            {"index": 0, "value": "md5", "description": ""},
            {"index": 1, "value": "sha1", "description": ""},
            {"index": 2, "value": "sha256", "description": ""},
            {"index": 3, "value": "filename", "description": ""},
            {"index": 4, "value": "pbd", "description": ""},
            {"index": 5, "value": "filename|md5", "description": ""},
            {"index": 6, "value": "filename|sha1", "description": ""},
            {"index": 7, "value": "filename|sha256", "description": ""},
            {"index": 8, "value": "ip-src", "description": ""},
            {"index": 9, "value": "ip-dst", "description": ""},
            {"index": 10, "value": "hostname", "description": ""},
            {"index": 11, "value": "domain", "description": ""},
            {"index": 12, "value": "domain|ip", "description": ""},
            {"index": 13, "value": "email-src", "description": ""},
            {"index": 14, "value": "eppn", "description": ""},
            {"index": 15, "value": "email-dst", "description": ""},
            {"index": 16, "value": "email-subject", "description": ""},
            {"index": 17, "value": "email-attachment", "description": ""},
            {"index": 18, "value": "email-body", "description": ""},
            {"index": 19, "value": "float", "description": ""},
            {"index": 20, "value": "url", "description": ""},
            {"index": 21, "value": "http-method", "description": ""},
            {"index": 22, "value": "user-agent", "description": ""},
            {"index": 23, "value": "ja3-fingerprint-md5", "description": ""},
            {"index": 24, "value": "hassh-md5", "description": ""},
            {"index": 25, "value": "hasshserver-md5", "description": ""},
            {"index": 26, "value": "reg-key", "description": ""},
            {"index": 27, "value": "regkey|value", "description": ""},
            {"index": 28, "value": "AS", "description": ""},
            {"index": 29, "value": "snort", "description": ""},
            {"index": 30, "value": "bro", "description": ""},
            {"index": 31, "value": "zeek", "description": ""},
            {"index": 32, "value": "community-id", "description": ""},
            {"index": 33, "value": "pattern-in-traffic", "description": ""},
            {"index": 34, "value": "pattern-in-memory", "description": ""},
            {"index": 35, "value": "yara", "description": ""},
            {"index": 36, "value": "stix2-pattern", "description": ""},
            {"index": 37, "value": "sigma", "description": ""},
            {"index": 38, "value": "gene", "description": ""},
            {"index": 39, "value": "kusto-query", "description": ""},
            {"index": 40, "value": "mime-type", "description": ""},
            {"index": 41, "value": "identity-card-number", "description": ""},
            {"index": 42, "value": "cookie", "description": ""},
            {"index": 43, "value": "vulnerability", "description": ""},
            {"index": 44, "value": "weakness", "description": ""},
            {"index": 45, "value": "link", "description": ""},
            {"index": 46, "value": "comment", "description": ""},
            {"index": 47, "value": "text", "description": ""},
            {"index": 48, "value": "hex", "description": ""},
            {"index": 49, "value": "other", "description": ""},
            {"index": 50, "value": "named pipe", "description": ""},
            {"index": 51, "value": "mutex", "description": ""},
            {"index": 52, "value": "target-user", "description": ""},
            {"index": 53, "value": "target-email", "description": ""},
            {"index": 54, "value": "target-machine", "description": ""},
            {"index": 55, "value": "target-org", "description": ""},
            {"index": 56, "value": "target-location", "description": ""},
            {"index": 57, "value": "target-external", "description": ""},
            {"index": 58, "value": "btc", "description": ""},
            {"index": 59, "value": "dash", "description": ""},
            {"index": 60, "value": "xmr", "description": ""},
            {"index": 61, "value": "iban", "description": ""},
            {"index": 62, "value": "bic", "description": ""},
            {"index": 63, "value": "bank-account-nr", "description": ""},
            {"index": 64, "value": "aba-rtn", "description": ""},
            {"index": 65, "value": "bin", "description": ""},
            {"index": 66, "value": "cc-number", "description": ""},
            {"index": 67, "value": "prtn", "description": ""},
            {"index": 68, "value": "phone-number", "description": ""},
            {"index": 69, "value": "threat-actor", "description": ""},
            {"index": 70, "value": "campaign-name", "description": ""},
            {"index": 71, "value": "campaign-id", "description": ""},
            {"index": 72, "value": "malware-type", "description": ""},
            {"index": 73, "value": "uri", "description": ""},
            {"index": 74, "value": "authentihash", "description": ""},
            {"index": 75, "value": "ssdeep", "description": ""},
            {"index": 76, "value": "implash", "description": ""},
            {"index": 77, "value": "pahash", "description": ""},
            {"index": 78, "value": "impfuzzy", "description": ""},
            {"index": 79, "value": "sha224", "description": ""},
            {"index": 80, "value": "sha384", "description": ""},
            {"index": 81, "value": "sha512", "description": ""},
            {"index": 82, "value": "sha512/224", "description": ""},
            {"index": 83, "value": "sha512/256", "description": ""},
            {"index": 84, "value": "tlsh", "description": ""},
            {"index": 85, "value": "cdhash", "description": ""},
            {"index": 86, "value": "filename|authentihash", "description": ""},
            {"index": 87, "value": "filename|ssdeep", "description": ""},
            {"index": 88, "value": "filename|implash", "description": ""},
            {"index": 89, "value": "filename|impfuzzy", "description": ""},
            {"index": 90, "value": "filename|pehash", "description": ""},
            {"index": 91, "value": "filename|sha224", "description": ""},
            {"index": 92, "value": "filename|sha384", "description": ""},
            {"index": 93, "value": "filename|sha512", "description": ""},
            {"index": 94, "value": "filename|sha512/224", "description": ""},
            {"index": 95, "value": "filename|sha512/256", "description": ""},
            {"index": 96, "value": "filename|tlsh", "description": ""},
            {"index": 97, "value": "windows-scheduled-task", "description": ""},
            {"index": 98, "value": "windows-service-name", "description": ""},
            {"index": 99, "value": "windows-service-displayname", "description": ""},
            {"index": 100, "value": "whois-registrant-email", "description": ""},
            {"index": 101, "value": "whois-registrant-phone", "description": ""},
            {"index": 102, "value": "whois-registrant-name", "description": ""},
            {"index": 103, "value": "whois-registrant-org", "description": ""},
            {"index": 104, "value": "whois-registrar", "description": ""},
            {"index": 105, "value": "whois-creation-date", "description": ""},
            {"index": 106, "value": "x509-fingerprint-sha1", "description": ""},
            {"index": 107, "value": "x509-fingerprint-md5", "description": ""},
            {"index": 108, "value": "x509-fingerprint-sha256", "description": ""},
            {"index": 109, "value": "dns-soa-email", "description": ""},
            {"index": 110, "value": "size-in-bytes", "description": ""},
            {"index": 111, "value": "counter", "description": ""},
            {"index": 112, "value": "datetime", "description": ""},
            {"index": 113, "value": "cpe", "description": ""},
            {"index": 114, "value": "port", "description": ""},
            {"index": 115, "value": "ip-dist|port", "description": ""},
            {"index": 116, "value": "ip-src|port", "description": ""},
            {"index": 117, "value": "hostname|port", "description": ""},
            {"index": 118, "value": "mac-address", "description": ""},
            {"index": 119, "value": "mac-eui-64", "description": ""},
            {"index": 120, "value": "email-dst-display-name", "description": ""},
            {"index": 121, "value": "email-src-display-name", "description": ""},
            {"index": 122, "value": "email-header", "description": ""},
            {"index": 123, "value": "email-reply-to", "description": ""},
            {"index": 124, "value": "email-x-mailer", "description": ""},
            {"index": 125, "value": "email-mime-boundary", "description": ""},
            {"index": 126, "value": "email-thread-index", "description": ""},
            {"index": 127, "value": "email-message-id", "description": ""},
            {"index": 128, "value": "github-username", "description": ""},
            {"index": 129, "value": "github-repository", "description": ""},
            {"index": 130, "value": "githzb-organisation", "description": ""},
            {"index": 131, "value": "jabber-id", "description": ""},
            {"index": 132, "value": "twitter-id", "description": ""},
            {"index": 133, "value": "first-name", "description": ""},
            {"index": 134, "value": "middle-name", "description": ""},
            {"index": 135, "value": "last-name", "description": ""},
            {"index": 136, "value": "date-of-birth", "description": ""},
            {"index": 137, "value": "gender", "description": ""},
            {"index": 138, "value": "passport-number", "description": ""},
            {"index": 139, "value": "passport-country", "description": ""},
            {"index": 140, "value": "passport-expiration", "description": ""},
            {"index": 141, "value": "redress-number", "description": ""},
            {"index": 142, "value": "nationality", "description": ""},
            {"index": 143, "value": "visa-number", "description": ""},
            {"index": 144, "value": "issue-date-of-the-visa", "description": ""},
            {"index": 145, "value": "primary-residence", "description": ""},
            {"index": 146, "value": "country-of-residence", "description": ""},
            {"index": 147, "value": "special-service-request", "description": ""},
            {"index": 148, "value": "frequent-flyer-number", "description": ""},
            {"index": 149, "value": "travel-details", "description": ""},
            {"index": 150, "value": "payments-details", "description": ""},
            {
                "index": 151,
                "value": "place-port-of-original-embarkation",
                "description": "",
            },
            {
                "index": 152,
                "value": "passenger-name-record-locator-number",
                "description": "",
            },
            {"index": 153, "value": "mobile-application-id", "description": ""},
            {"index": 154, "value": "chrome-extension-id", "description": ""},
            {"index": 155, "value": "cortex", "description": ""},
            {"index": 156, "value": "boolean", "description": ""},
            {"index": 157, "value": "anonymised", "description": ""},
        ]
        attr = {
            **base_attr,
            **{
                "name": "MISP Attribute Type",
                "description": "Combo box for MISP attribute type",
                "type": "ENUM",
                "attribute_enums": attr_enum,
                "attribute_enums_total_count": len(attr_enum),
            },
        }
        Attribute.add_attribute(attr)

    if not db.session.query(Attribute).filter_by(name="MISP Attribute Distribution").first():
        attr_enum = [
            {"index": 0, "value": "Your organisation only", "description": ""},
            {"index": 1, "value": "This community only", "description": ""},
            {"index": 2, "value": "Connected communities", "description": ""},
            {"index": 3, "value": "All communities", "description": ""},
            {"index": 4, "value": "Inherit event", "description": ""},
        ]
        attr = {
            **base_attr,
            **{
                "name": "MISP Attribute Distribution",
                "description": "Combo box for MISP attribute type",
                "type": "ENUM",
                "attribute_enums": attr_enum,
                "attribute_enums_total_count": len(attr_enum),
            },
        }
        Attribute.add_attribute(attr)


def pre_seed_report_items():
    from core.model.report_item_type import (
        ReportItemType,
        AttributeGroupItem,
        AttributeGroup,
    )
    from core.model.attribute import Attribute

    if not db.session.query(ReportItemType).filter_by(title="Vulnerability Report").first():
        report_item_type = ReportItemType(None, "Vulnerability Report", "Basic report type", [])
        db.session.add(report_item_type)
        db.session.commit()

        cvss_item = AttributeGroupItem(
            None,
            "CVSS",
            "",
            0,
            1,
            1,
            db.session.query(Attribute).filter_by(name="CVSS").first().id,
        )
        tlp_item = AttributeGroupItem(
            None,
            "TLP",
            "",
            1,
            1,
            1,
            db.session.query(Attribute).filter_by(name="TLP").first().id,
        )
        confidentiality_item = AttributeGroupItem(
            None,
            "Confidentiality",
            "",
            2,
            1,
            1,
            db.session.query(Attribute).filter_by(name="Confidentiality").first().id,
        )
        description_item = AttributeGroupItem(
            None,
            "Description",
            "",
            3,
            1,
            1,
            db.session.query(Attribute).filter_by(name="Text Area").first().id,
        )
        exposure_date_item = AttributeGroupItem(
            None,
            "Exposure Date",
            "",
            4,
            1,
            1,
            db.session.query(Attribute).filter_by(name="Date").first().id,
        )
        update_date_item = AttributeGroupItem(
            None,
            "Update Date",
            "",
            5,
            1,
            1,
            db.session.query(Attribute).filter_by(name="Date").first().id,
        )
        cve_item = AttributeGroupItem(
            None,
            "CVE",
            "",
            6,
            0,
            1000,
            db.session.query(Attribute).filter_by(name="CVE").first().id,
        )
        impact_item = AttributeGroupItem(
            None,
            "Impact",
            "",
            7,
            0,
            1000,
            db.session.query(Attribute).filter_by(name="Impact").first().id,
        )

        group1 = AttributeGroup(
            None,
            "Vulnerability",
            "",
            0,
            "",
            0,
            [
                cvss_item,
                tlp_item,
                confidentiality_item,
                description_item,
                exposure_date_item,
                update_date_item,
                cve_item,
                impact_item,
            ],
        )
        db.session.add(group1)
        db.session.commit()

        affected_systems_item = AttributeGroupItem(
            None,
            "Affected systems",
            "",
            0,
            0,
            1000,
            db.session.query(Attribute).filter_by(name="CPE").first().id,
        )
        ioc_item = AttributeGroupItem(
            None,
            "IOC",
            "",
            1,
            0,
            1000,
            db.session.query(Attribute).filter_by(name="Text").first().id,
        )
        recommendations_item = AttributeGroupItem(
            None,
            "Recommendations",
            "",
            2,
            1,
            1,
            db.session.query(Attribute).filter_by(name="Text Area").first().id,
        )
        group2 = AttributeGroup(
            None,
            "Identify and Act",
            "",
            0,
            "",
            0,
            [affected_systems_item, ioc_item, recommendations_item],
        )
        db.session.add(group2)
        db.session.commit()

        links_item = AttributeGroupItem(
            None,
            "Links",
            "",
            0,
            0,
            1000,
            db.session.query(Attribute).filter_by(name="Text").first().id,
        )
        db.session.add(AttributeGroup(None, "Resources", "", 0, "", 0, [links_item]))
        db.session.commit()

    if not db.session.query(ReportItemType).filter_by(title="MISP Report").first():
        report_item_type = ReportItemType(None, "MISP Report", "MISP report type", [])
        db.session.add(report_item_type)
        db.session.commit()

        db.session.add(
            AttributeGroupItem(
                None,
                "Event distribution",
                "",
                0,
                1,
                1,
                db.session.query(Attribute).filter_by(name="Text").first().id,
            )
        )
        db.session.add(
            AttributeGroupItem(
                None,
                "Event threat level",
                "",
                1,
                1,
                1,
                db.session.query(Attribute).filter_by(name="Text").first().id,
            )
        )
        db.session.add(
            AttributeGroupItem(
                None,
                "Event analysis",
                "",
                2,
                1,
                1,
                db.session.query(Attribute).filter_by(name="Text").first().id,
            )
        )
        db.session.add(
            AttributeGroupItem(
                None,
                "Event info",
                "",
                2,
                1,
                1,
                db.session.query(Attribute).filter_by(name="Text").first().id,
            )
        )

        group4 = AttributeGroup(None, "Event", "", None, None, 0, [])
        db.session.add(group4)
        db.session.commit()

        db.session.add(
            AttributeGroupItem(
                None,
                "Attribute category",
                "",
                0,
                1,
                1,
                db.session.query(Attribute).filter_by(name="Text").first().id,
            )
        )
        db.session.add(
            AttributeGroupItem(
                None,
                "Attribute type",
                "",
                1,
                1,
                1,
                db.session.query(Attribute).filter_by(name="Text").first().id,
            )
        )
        db.session.add(
            AttributeGroupItem(
                None,
                "Attribute distribution",
                "",
                2,
                1,
                1,
                db.session.query(Attribute).filter_by(name="Text").first().id,
            )
        )
        db.session.add(
            AttributeGroupItem(
                None,
                "Attribute value",
                "",
                3,
                1,
                1,
                db.session.query(Attribute).filter_by(name="Text Area").first().id,
            )
        )
        db.session.add(
            AttributeGroupItem(
                None,
                "Attribute contextual comment",
                "",
                4,
                1,
                1,
                db.session.query(Attribute).filter_by(name="Text").first().id,
            )
        )
        db.session.add(
            AttributeGroupItem(
                None,
                "Attribute additional information",
                "",
                5,
                1,
                1,
                db.session.query(Attribute).filter_by(name="Additional Data").first().id,
            )
        )
        db.session.add(
            AttributeGroupItem(
                None,
                "First seen date",
                "",
                6,
                1,
                1,
                db.session.query(Attribute).filter_by(name="Date").first().id,
            )
        )
        db.session.add(
            AttributeGroupItem(
                None,
                "Last seen date",
                "",
                7,
                1,
                1,
                db.session.query(Attribute).filter_by(name="Date").first().id,
            )
        )
        group5 = AttributeGroup(None, "Attribute", "", None, None, 0, [])
        db.session.add(group5)

        db.session.commit()


def pre_seed_wordlists():
    from core.model.word_list import WordList

    if not WordList.find_by_name(name="Default EN stop list"):
        WordList.add_new(
            {
                "id": -1,
                "name": "Default EN stop list",
                "description": "English stop-word list packed with the standard Taranis NG installation.",
                "categories": [
                    {
                        "name": "Default EN stop list",
                        "description": "Source: https://www.maxqda.de/hilfe-mx20-dictio/stopp-listen",
                        "entries": [],
                        "link": "https://raw.githubusercontent.com/SK-CERT/Taranis-NG/main/resources/wordlists/en_complete.csv",
                    }
                ],
                "use_for_stop_words": True,
            }
        )

    if not WordList.find_by_name(name="Default SK stop list"):
        WordList.add_new(
            {
                "id": -1,
                "name": "Default SK stop list",
                "description": "Slovak stop-word list packed with the standard Taranis NG installation.",
                "categories": [
                    {
                        "name": "Default SK stop list",
                        "description": "Source: https://github.com/stopwords-iso/stopwords-sk/blob/master/stopwords-sk.txt",
                        "entries": [],
                        "link": "https://raw.githubusercontent.com/SK-CERT/Taranis-NG/main/resources/wordlists/sk_complete.csv",
                    }
                ],
                "use_for_stop_words": True,
            }
        )

    if not WordList.find_by_name(name="Default highlighting wordlist"):
        WordList.add_new(
            {
                "id": -1,
                "name": "Default highlighting wordlist",
                "description": "Default highlighting list packed with the standard Taranis NG installation.",
                "categories": [
                    {
                        "name": "Default highlighting wordlist",
                        "description": "Source: https://www.allot.com/100-plus-cybersecurity-terms-definitions/",
                        "entries": [],
                        "link": "https://raw.githubusercontent.com/SK-CERT/Taranis-NG/main/resources/wordlists/highlighting.csv",
                    }
                ],
                "use_for_stop_words": True,
            }
        )


def pre_seed_default_user():
    from werkzeug.security import generate_password_hash
    from core.model.organization import Organization
    from core.model.user import User

    admin_organization = Organization.find(1)
    if not admin_organization:
        Organization.add_new(
            {
                "id": 1,
                "name": "The Earth",
                "description": "Earth is the third planet from the Sun and the only astronomical object known to harbor life.",
                "address": {"street": "29 Arlington Avenue", "city": "Islington, London", "zip": "N1 7BE", "country": "United Kingdom"},
            }
        )

    if not User.find(username="admin"):
        User.add_new(
            {
                "id": -1,
                "username": "admin",
                "name": "Arthur Dent",
                "roles": [
                    {
                        "id": "1",
                    },
                ],
                "permissions": [],
                "organizations": [
                    {
                        "id": 1,
                    },
                ],
                "password": generate_password_hash("admin", method="sha256"),
            }
        )

    if not Organization.find(2):
        Organization.add_new(
            {
                "id": 2,
                "name": "The Clacks",
                "description": "A network infrastructure of Semaphore Towers, that operate in a similar fashion to telegraph.",
                "address": {
                    "street": "Cherry Tree Rd",
                    "city": "Beaconsfield, Buckinghamshire",
                    "zip": "HP9 1BH",
                    "country": "United Kingdom",
                },
            }
        )

    if not User.find(username="user"):
        User.add_new(
            {
                "id": -1,
                "username": "user",
                "name": "Terry Pratchett",
                "roles": [
                    {
                        "id": "2",
                    },
                ],
                "permissions": [],
                "organizations": [
                    {
                        "id": 2,
                    },
                ],
                "password": generate_password_hash("user", method="sha256"),
            }
        )



import urllib
from flask import redirect, make_response
from flask_restful import Resource, reqparse, request, ResponseBase
from flask_jwt_extended import jwt_required

from core.config import Config
from core.managers import auth_manager
from core.managers.log_manager import logger
from core.managers.auth_manager import no_auth


class Login(Resource):
    @no_auth
    def get(self):
        response = auth_manager.authenticate(None)
        if not isinstance(response, ResponseBase) and "access_token" in response:
            goto_url = request.args.get(key="gotoUrl", default="/")
            redirect_response = make_response(redirect(goto_url))
            redirect_response.set_cookie("jwt", response["access_token"])
            return redirect_response
        return response

    def post(self):
        parser = reqparse.RequestParser()
        logger.log_debug(auth_manager.get_required_credentials())
        for credential in auth_manager.get_required_credentials():
            parser.add_argument(credential, location=["form", "values", "json"])
        credentials = parser.parse_args()
        return auth_manager.authenticate(credentials)


class Refresh(Resource):
    @jwt_required()
    def get(self):
        return auth_manager.refresh(auth_manager.get_user_from_jwt())


class Logout(Resource):
    @no_auth
    def get(self):
        token = request.args["jwt"] if "jwt" in request.args else None
        response = auth_manager.logout(token)

        if not isinstance(response, ResponseBase) and "gotoUrl" in request.args:
            goto_url = request.args["gotoUrl"]
            url = Config.OPENID_LOGOUT_URL.replace("GOTO_URL", urllib.parse.quote(goto_url)) if Config.OPENID_LOGOUT_URL else goto_url
            return redirect(url)

        return response


def initialize(api):
    api.add_resource(Login, "/api/v1/auth/login")
    api.add_resource(Refresh, "/api/v1/auth/refresh")
    api.add_resource(Logout, "/api/v1/auth/logout")



from flask import request
import re
import os

from core.config import Config
from shared.log import TaranisLogger


class Logger(TaranisLogger):
    def resolve_ip_address(self):
        headers_list = request.headers.getlist("X-Forwarded-For")
        return headers_list[0] if headers_list else request.remote_addr

    def resolve_method(self):
        return request.method

    def resolve_resource(self):
        fp_len = len(request.full_path)
        return request.full_path[: fp_len - 1] if request.full_path[fp_len - 1] == "?" else request.full_path

    def resolve_data(self):
        if "application/json" not in request.headers.get("Content-Type", ""):
            return ""
        if request.data is None:
            return ""

        return str(request.data)[:4096].replace("\\r", "").replace("\\n", "").replace(" ", "")[2:-1]

    def generate_escaped_data(self):
        data = self.resolve_data()
        return re.escape(data)[:4096]

    def rollback_and_store_to_db(self, log_data):
        from core.managers.db_manager import db

        db.session.rollback()

        # from core.model.log_record import LogRecord
        # LogRecord.store(log_data)

    def store_access_error_activity(self, user, activity_detail):
        self.store_user_auth_error_activity(user, activity_detail, "TARANIS NG Access Error: (%s)")

    def store_data_error_activity(self, user, activity_detail):
        self.store_user_auth_error_activity(user, activity_detail, "TARANIS NG Data Error: (%s)")

    def store_data_error_activity_no_user(self, activity_detail):
        log_data = {
            "ip_address": self.resolve_ip_address(),
            "user_id": None,
            "user_name": None,
            "system_id": None,
            "system_name": None,
            "module_id": self.module,
            "activity_type": None,
            "activity_resource": self.resolve_resource(),
            "activity_detail": activity_detail,
            "activity_method": self.resolve_method(),
            "activity_data": self.resolve_data(),
        }

        self.rollback_and_store_to_db(log_data)
        self.logger.critical("TARANIS NG Public Access Data Error: (%s)", log_data)

    def store_auth_error_activity(self, activity_detail):
        log_data = {
            "ip_address": self.resolve_ip_address(),
            "user_id": None,
            "user_name": None,
            "system_id": None,
            "system_name": None,
            "module_id": self.module,
            "activity_type": None,
            "activity_resource": self.resolve_resource(),
            "activity_detail": activity_detail,
            "activity_method": self.resolve_method(),
            "activity_data": self.resolve_data(),
        }

        self.rollback_and_store_to_db(log_data)
        self.logger.critical("TARANIS NG Auth Error: (%s)", log_data)

    def store_user_auth_error_activity(self, user, activity_detail, message):
        log_data = {
            "ip_address": self.resolve_ip_address(),
            "user_id": user.id,
            "user_name": user.name,
            "system_id": None,
            "system_name": None,
            "module_id": self.module,
            "activity_type": None,
            "activity_resource": self.resolve_resource(),
            "activity_detail": activity_detail,
            "activity_method": self.resolve_method(),
            "activity_data": self.resolve_data(),
        }

        self.rollback_and_store_to_db(log_data)
        self.logger.critical(message, log_data)

    def store_system_error_activity(self, system_id, system_name, activity_type, activity_detail):
        log_data = {
            "ip_address": self.resolve_ip_address(),
            "user_id": None,
            "user_name": None,
            "system_id": system_id,
            "system_name": system_name,
            "module_id": self.module,
            "activity_type": activity_type,
            "activity_resource": self.resolve_resource(),
            "activity_detail": activity_detail,
            "activity_method": self.resolve_method(),
            "activity_data": self.resolve_data(),
        }

        self.rollback_and_store_to_db(log_data)
        self.logger.critical("TARANIS NG System Critical: (%s)", log_data)

    def store_system_activity(self, system_id, system_name, activity_type, activity_detail):
        pass

    def store_activity(self, activity_type, activity_detail):
        pass

    def store_user_activity(self, user, activity_type, activity_detail):
        self.logger.debug(f"User: {user.name} activity_type: {activity_type} activity_detail: {activity_detail}")


gunicorn = "gunicorn" in os.environ.get("SERVER_SOFTWARE", "")
logger = Logger(module=Config.MODULE_ID, colored=Config.COLORED_LOGS, debug=Config.DEBUG, gunicorn=gunicorn, syslog_address=None)



from marshmallow import fields, post_load
from sqlalchemy import orm, func, or_, and_
import sqlalchemy
from sqlalchemy.sql.expression import cast

from core.managers.db_manager import db
from core.model.acl_entry import ACLEntry
from core.model.attribute import Attribute
from shared.schema.acl_entry import ItemType
from shared.schema.report_item_type import (
    AttributeGroupItemSchema,
    AttributeGroupBaseSchema,
    ReportItemTypeBaseSchema,
    ReportItemTypePresentationSchema,
)


class NewAttributeGroupItemSchema(AttributeGroupItemSchema):
    attribute_id = fields.Integer()

    @post_load
    def make_attribute_group_item(self, data, **kwargs):
        return AttributeGroupItem(**data)


class AttributeGroupItem(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    title = db.Column(db.String())
    description = db.Column(db.String())

    index = db.Column(db.Integer)
    min_occurrence = db.Column(db.Integer)
    max_occurrence = db.Column(db.Integer)

    attribute_group_id = db.Column(db.Integer, db.ForeignKey("attribute_group.id"))
    attribute_group = db.relationship("AttributeGroup")

    attribute_id = db.Column(db.Integer, db.ForeignKey("attribute.id"))
    attribute = db.relationship("Attribute")

    def __init__(
        self,
        id,
        title,
        description,
        index,
        min_occurrence,
        max_occurrence,
        attribute_id,
    ):
        self.id = id if id is not None and id != -1 else None
        self.title = title
        self.description = description
        self.index = index
        self.min_occurrence = min_occurrence
        self.max_occurrence = max_occurrence
        self.attribute_id = attribute_id

    @classmethod
    def find(cls, id):
        return cls.query.get(id)

    @staticmethod
    def sort(attribute_group_item):
        return attribute_group_item.index


class NewAttributeGroupSchema(AttributeGroupBaseSchema):
    attribute_group_items = fields.Nested("NewAttributeGroupItemSchema", many=True)

    @post_load
    def make_attribute_group(self, data, **kwargs):
        return AttributeGroup(**data)


class AttributeGroup(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    title = db.Column(db.String())
    description = db.Column(db.String())

    section = db.Column(db.Integer)
    section_title = db.Column(db.String())
    index = db.Column(db.Integer)

    report_item_type_id = db.Column(db.Integer, db.ForeignKey("report_item_type.id"))
    report_item_type = db.relationship("ReportItemType")

    attribute_group_items = db.relationship(
        "AttributeGroupItem",
        back_populates="attribute_group",
        cascade="all, delete-orphan",
    )

    def __init__(
        self,
        id,
        title,
        description,
        section,
        section_title,
        index,
        attribute_group_items,
    ):
        if id is not None and id != -1:
            self.id = id
        else:
            self.id = None

        self.title = title
        self.description = description
        self.section = section
        self.section_title = section_title
        self.index = index
        self.attribute_group_items = attribute_group_items

    @orm.reconstructor
    def reconstruct(self):
        self.attribute_group_items.sort(key=AttributeGroupItem.sort)

    @staticmethod
    def sort(attribute_group):
        return attribute_group.index

    def update(self, updated_attribute_group):
        self.title = updated_attribute_group.title
        self.description = updated_attribute_group.description
        self.section = updated_attribute_group.section
        self.section_title = updated_attribute_group.section_title
        self.index = updated_attribute_group.index

        for updated_attribute_group_item in updated_attribute_group.attribute_group_items:
            found = False
            for attribute_group_item in self.attribute_group_items:
                if updated_attribute_group_item.id == attribute_group_item.id:
                    attribute_group_item.title = updated_attribute_group_item.title
                    attribute_group_item.description = updated_attribute_group_item.description
                    attribute_group_item.index = updated_attribute_group_item.index
                    attribute_group_item.min_occurrence = updated_attribute_group_item.min_occurrence
                    attribute_group_item.max_occurrence = updated_attribute_group_item.max_occurrence
                    attribute_group_item.attribute_id = updated_attribute_group_item.attribute_id
                    found = True
                    break

            if found is False:
                updated_attribute_group_item.attribute_group = None
                self.attribute_group_items.append(updated_attribute_group_item)

        for attribute_group_item in self.attribute_group_items[:]:
            found = False
            for updated_attribute_group_item in updated_attribute_group.attribute_group_items:
                if updated_attribute_group_item.id == attribute_group_item.id:
                    found = True
                    break

            if found is False:
                self.attribute_group_items.remove(attribute_group_item)


class NewReportItemTypeSchema(ReportItemTypeBaseSchema):
    attribute_groups = fields.Nested("NewAttributeGroupSchema", many=True)

    @post_load
    def make_report_item_type(self, data, **kwargs):
        return ReportItemType(**data)


class ReportItemType(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    title = db.Column(db.String())
    description = db.Column(db.String())

    attribute_groups = db.relationship(
        "AttributeGroup",
        back_populates="report_item_type",
        cascade="all, delete-orphan",
    )

    def __init__(self, id, title, description, attribute_groups):
        self.id = None
        self.title = title
        self.description = description
        self.attribute_groups = attribute_groups
        self.subtitle = ""
        self.tag = ""

    @orm.reconstructor
    def reconstruct(self):
        self.subtitle = self.description
        self.tag = "mdi-file-table-outline"
        self.attribute_groups.sort(key=AttributeGroup.sort)

    @classmethod
    def find(cls, id):
        return cls.query.get(id)

    @classmethod
    def get_all(cls):
        return cls.query.order_by(ReportItemType.title).all()

    @classmethod
    def allowed_with_acl(cls, report_item_type_id, user, see, access, modify):

        query = db.session.query(ReportItemType.id).distinct().group_by(ReportItemType.id).filter(ReportItemType.id == report_item_type_id)

        query = query.outerjoin(
            ACLEntry,
            and_(
                cast(ReportItemType.id, sqlalchemy.String) == ACLEntry.item_id,
                ACLEntry.item_type == ItemType.REPORT_ITEM_TYPE,
            ),
        )

        query = ACLEntry.apply_query(query, user, see, access, modify)

        return query.scalar() is not None

    @classmethod
    def get(cls, search, user, acl_check):
        query = cls.query.distinct().group_by(ReportItemType.id)

        if acl_check is True:
            query = query.outerjoin(
                ACLEntry,
                and_(
                    cast(ReportItemType.id, sqlalchemy.String) == ACLEntry.item_id,
                    ACLEntry.item_type == ItemType.REPORT_ITEM_TYPE,
                ),
            )
            query = ACLEntry.apply_query(query, user, True, False, False)

        if search is not None:
            search_string = "%" + search.lower() + "%"
            query = query.filter(
                or_(
                    func.lower(ReportItemType.title).like(search_string),
                    func.lower(ReportItemType.description).like(search_string),
                )
            )

        return query.order_by(db.asc(ReportItemType.title)).all(), query.count()

    @classmethod
    def get_all_json(cls, search, user, acl_check):
        report_item_types, count = cls.get(search, user, acl_check)
        for report_item_type in report_item_types:
            for attribute_group in report_item_type.attribute_groups:
                for attribute_group_item in attribute_group.attribute_group_items:
                    attribute_group_item.attribute.attribute_enums = Attribute.get_enums(attribute_group_item.attribute)

        report_item_type_schema = ReportItemTypePresentationSchema(many=True)
        return {
            "total_count": count,
            "items": report_item_type_schema.dump(report_item_types),
        }

    @classmethod
    def add_report_item_type(cls, report_item_type_data):
        report_item_type_schema = NewReportItemTypeSchema()
        report_item_type = report_item_type_schema.load(report_item_type_data)
        db.session.add(report_item_type)
        db.session.commit()

    @classmethod
    def update(cls, report_type_id, data):
        schema = NewReportItemTypeSchema()
        updated_report_type = schema.load(data)
        report_type = cls.query.get(report_type_id)
        report_type.title = updated_report_type.title
        report_type.description = updated_report_type.description

        for updated_attribute_group in updated_report_type.attribute_groups:
            found = False
            for attribute_group in report_type.attribute_groups:
                if updated_attribute_group.id is not None and updated_attribute_group.id == attribute_group.id:
                    attribute_group.update(updated_attribute_group)
                    found = True
                    break

            if found is False:
                updated_attribute_group.report_item_type = None
                report_type.attribute_groups.append(updated_attribute_group)

        for attribute_group in report_type.attribute_groups[:]:
            found = False
            for updated_attribute_group in updated_report_type.attribute_groups:
                if updated_attribute_group.id == attribute_group.id:
                    found = True
                    break

            if found is False:
                report_type.attribute_groups.remove(attribute_group)

        db.session.commit()

    @classmethod
    def delete_report_item_type(cls, id):
        report_item_type = cls.query.get(id)
        db.session.delete(report_item_type)
        db.session.commit()



from marshmallow import post_load

from core.managers.db_manager import db
from shared.schema.address import AddressSchema


class NewAddressSchema(AddressSchema):
    @post_load
    def make(self, data, **kwargs):
        return Address(**data)


class Address(db.Model):
    id = db.Column(db.Integer, primary_key=True)

    street = db.Column(db.String())
    city = db.Column(db.String())
    zip = db.Column(db.String())
    country = db.Column(db.String())

    def __init__(self, street, city, zip, country):
        self.id = None
        self.street = street
        self.city = city
        self.zip = zip
        self.country = country



import requests


class PublishersApi:
    def __init__(self, api_url, api_key):
        self.api_url = api_url
        if self.api_url.endswith("/"):
            self.api_url = self.api_url[:-1]
        self.api_key = api_key
        self.headers = {"Authorization": f"Bearer {self.api_key}"}

    def get_publishers_info(self):
        response = requests.get(f"{self.api_url}/api/v1/publishers", headers=self.headers)

        return response.json(), response.status_code

    def publish(self, data):
        response = requests.post(f"{self.api_url}/api/v1/publishers", json=data, headers=self.headers)

        return response.json(), response.status_code



from marshmallow import post_load

from core.managers.db_manager import db
from shared.schema.parameter import ParameterType, ParameterSchema


class NewParameterSchema(ParameterSchema):
    @post_load
    def make_parameter(self, data, **kwargs):
        return Parameter(**data)


class Parameter(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    key = db.Column(db.String(), nullable=False)
    name = db.Column(db.String(), nullable=False)
    description = db.Column(db.String())
    type = db.Column(db.Enum(ParameterType))

    def __init__(self, id, key, name, description, type):
        self.id = None
        self.key = key
        self.name = name
        self.description = description
        self.type = type



from flask_restful import Resource

from core.managers.auth_manager import no_auth


class IsAlive(Resource):
    @no_auth
    def get(self):
        return {"isalive": True}


def initialize(api):
    api.add_resource(IsAlive, "/api/v1/isalive", "/")



from sqlalchemy import func, or_

from core.managers.db_manager import db
from shared.schema.role import PermissionSchema


class Permission(db.Model):
    id = db.Column(db.String, primary_key=True)
    name = db.Column(db.String(), unique=True, nullable=False)
    description = db.Column(db.String())

    roles = db.relationship("Role", secondary="role_permission")

    @classmethod
    def find(cls, permission_id):
        permission = cls.query.get(permission_id)
        return permission

    @classmethod
    def add(cls, id, name, description):
        permission = cls.find(id)
        if permission is None:
            permission = Permission()
            permission.id = id
            permission.name = name
            permission.description = description
            db.session.add(permission)
        else:
            permission.name = name
            permission.description = description
        db.session.commit()

    @classmethod
    def get_all(cls):
        return cls.query.order_by(db.asc(Permission.id)).all()

    @classmethod
    def get(cls, search):
        query = cls.query

        if search is not None:
            search_string = f"%{search.lower()}%"
            query = query.filter(
                or_(
                    func.lower(Permission.name).like(search_string),
                    func.lower(Permission.description).like(search_string),
                )
            )

        return query.order_by(db.asc(Permission.id)).all(), query.count()

    @classmethod
    def get_all_json(cls, search):
        permissions, count = cls.get(search)
        permissions_schema = PermissionSchema(many=True)
        return {"total_count": count, "items": permissions_schema.dump(permissions)}



import uuid as uuid_generator
from datetime import datetime
from marshmallow import post_load, fields
from sqlalchemy import orm, func, or_, and_

from core.managers.db_manager import db
from core.model.osint_source import OSINTSource
from core.model.report_item_type import ReportItemType
from shared.schema.osint_source import OSINTSourceIdSchema
from shared.schema.remote import (
    RemoteAccessSchema,
    RemoteAccessPresentationSchema,
    RemoteNodeSchema,
    RemoteNodePresentationSchema,
)
from shared.schema.report_item_type import ReportItemTypeIdSchema


class NewRemoteAccessSchema(RemoteAccessSchema):
    osint_sources = fields.Nested(OSINTSourceIdSchema, many=True)
    report_item_types = fields.Nested(ReportItemTypeIdSchema, many=True)

    @post_load
    def make(self, data, **kwargs):
        return RemoteAccess(**data)


class RemoteAccess(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(), nullable=False)
    description = db.Column(db.String())

    enabled = db.Column(db.Boolean)
    connected = db.Column(db.Boolean, default=False)
    access_key = db.Column(db.String(), unique=True)

    osint_sources = db.relationship("OSINTSource", secondary="remote_access_osint_source")
    report_item_types = db.relationship("ReportItemType", secondary="remote_access_report_item_type")

    event_id = db.Column(db.String(64), unique=True)
    last_synced_news_items = db.Column(db.DateTime, default=datetime.now())
    last_synced_report_items = db.Column(db.DateTime, default=datetime.now())

    def __init__(
        self,
        id,
        name,
        description,
        enabled,
        access_key,
        osint_sources,
        report_item_types,
    ):
        self.id = None
        self.name = name
        self.description = description
        self.enabled = enabled
        self.access_key = access_key
        self.event_id = str(uuid_generator.uuid4())
        self.title = ""
        self.subtitle = ""
        self.tag = ""
        self.status = ""

        self.osint_sources = []
        for osint_source in osint_sources:
            self.osint_sources.append(OSINTSource.find(osint_source.id))

        self.report_item_types = []
        for report_item_type in report_item_types:
            self.report_item_types.append(ReportItemType.find(report_item_type.id))

    @orm.reconstructor
    def reconstruct(self):
        self.title = self.name
        self.subtitle = self.description
        self.tag = "mdi-remote-desktop"
        if self.enabled is False:
            self.status = "red"
        elif self.connected is True:
            self.status = "green"
        else:
            self.status = "orange"

    @classmethod
    def exists_by_access_key(cls, access_key):
        return db.session.query(db.exists().where(RemoteAccess.access_key == access_key)).scalar()

    @classmethod
    def find_by_access_key(cls, access_key):
        remote_access = cls.query.filter(RemoteAccess.access_key == access_key).scalar()
        return remote_access

    @classmethod
    def get(cls, search):
        query = cls.query

        if search is not None:
            search_string = "%" + search.lower() + "%"
            query = query.filter(
                or_(
                    func.lower(RemoteAccess.name).like(search_string),
                    func.lower(RemoteAccess.description).like(search_string),
                )
            )

        return query.order_by(db.asc(RemoteAccess.name)).all(), query.count()

    @classmethod
    def get_all_json(cls, search):
        remote_accesses, count = cls.get(search)
        schema = RemoteAccessPresentationSchema(many=True)
        return {"total_count": count, "items": schema.dump(remote_accesses)}

    @classmethod
    def get_relevant_for_news_items(cls, osint_source_ids):
        query = db.session.query(RemoteAccess.event_id).join(
            RemoteAccessOSINTSource,
            and_(
                RemoteAccessOSINTSource.remote_access_id == RemoteAccess.id,
                RemoteAccessOSINTSource.osint_source_id.in_(osint_source_ids),
            ),
        )

        response = query.all()
        ids = set()
        for rows in response:
            ids.add(rows[0])

        return list(ids)

    @classmethod
    def get_relevant_for_report_item(cls, report_type_id):
        query = db.session.query(RemoteAccess.event_id).join(
            RemoteAccessReportItemType,
            and_(
                RemoteAccessReportItemType.remote_access_id == RemoteAccess.id,
                RemoteAccessReportItemType.report_item_type_id == report_type_id,
            ),
        )

        response = query.all()
        ids = set()
        for rows in response:
            ids.add(rows[0])

        return list(ids)

    @classmethod
    def add(cls, data):
        schema = NewRemoteAccessSchema()
        remote_access = schema.load(data)
        db.session.add(remote_access)
        db.session.commit()

    @classmethod
    def delete(cls, remote_access_id):
        remote_access = cls.query.get(remote_access_id)
        db.session.delete(remote_access)
        db.session.commit()

    @classmethod
    def update(cls, remote_access_id, data):
        schema = NewRemoteAccessSchema()
        updated_remote_access = schema.load(data)
        remote_access = cls.query.get(remote_access_id)
        remote_access.name = updated_remote_access.name
        remote_access.description = updated_remote_access.description
        remote_access.osint_sources = updated_remote_access.osint_sources
        remote_access.report_item_types = updated_remote_access.report_item_types

        disconnect = False
        event_id = remote_access.event_id

        if remote_access.enabled and not updated_remote_access.enabled:
            remote_access.enabled = False
            if remote_access.connected:
                remote_access.connected = False
                disconnect = True
        else:
            remote_access.enabled = updated_remote_access.enabled

        if remote_access.access_key != updated_remote_access.access_key or not remote_access.enabled:
            disconnect = True
            remote_access.connected = False
            remote_access.event_id = str(uuid_generator.uuid4())

        remote_access.access_key = updated_remote_access.access_key
        db.session.commit()

        return event_id, disconnect

    @classmethod
    def connect(cls, access_key):
        remote_access = cls.query.filter(RemoteAccess.access_key == access_key).scalar()
        if remote_access.enabled:
            remote_access.connected = True
            db.session.commit()
            return {
                "event_id": remote_access.event_id,
                "last_synced_news_items": format(remote_access.last_synced_news_items),
                "last_synced_report_items": format(remote_access.last_synced_report_items),
                "news_items_provided": len(remote_access.osint_sources) > 0,
                "report_items_provided": len(remote_access.report_item_types) > 0,
            }
        else:
            return {"error": "unauthorized"}, 401

    @classmethod
    def disconnect(cls, access_key):
        remote_access = cls.query.filter(RemoteAccess.access_key == access_key).scalar()
        remote_access.connected = False
        db.session.commit()

    def update_news_items_sync(self, data):
        self.last_synced_news_items = datetime.strptime(data["last_sync_time"], "%Y-%m-%d %H:%M:%S.%f")
        db.session.commit()

    def update_report_items_sync(self, data):
        self.last_synced_report_items = datetime.strptime(data["last_sync_time"], "%Y-%m-%d %H:%M:%S.%f")
        db.session.commit()


class RemoteAccessOSINTSource(db.Model):
    remote_access_id = db.Column(db.Integer, db.ForeignKey("remote_access.id"), primary_key=True)
    osint_source_id = db.Column(db.String, db.ForeignKey("osint_source.id"), primary_key=True)


class RemoteAccessReportItemType(db.Model):
    remote_access_id = db.Column(db.Integer, db.ForeignKey("remote_access.id"), primary_key=True)
    report_item_type_id = db.Column(db.Integer, db.ForeignKey("report_item_type.id"), primary_key=True)


class NewRemoteNodeSchema(RemoteNodeSchema):
    @post_load
    def make(self, data, **kwargs):
        return RemoteNode(**data)


class RemoteNode(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(), nullable=False)
    description = db.Column(db.String())

    enabled = db.Column(db.Boolean)
    remote_url = db.Column(db.String())
    events_url = db.Column(db.String())
    access_key = db.Column(db.String())

    sync_news_items = db.Column(db.Boolean)
    osint_source_group_id = db.Column(db.String, db.ForeignKey("osint_source_group.id"))
    sync_report_items = db.Column(db.Boolean)

    event_id = db.Column(db.String(64), unique=True)
    last_synced_news_items = db.Column(db.DateTime)
    last_synced_report_items = db.Column(db.DateTime)

    def __init__(
        self,
        id,
        name,
        description,
        enabled,
        remote_url,
        events_url,
        access_key,
        sync_news_items,
        sync_report_items,
        osint_source_group_id,
    ):
        self.id = None
        self.name = name
        self.description = description
        self.remote_url = remote_url
        self.events_url = events_url
        self.enabled = enabled
        self.access_key = access_key
        self.sync_news_items = sync_news_items
        self.sync_report_items = sync_report_items
        self.osint_source_group_id = osint_source_group_id
        self.title = ""
        self.subtitle = ""
        self.tag = ""
        self.status = ""

    @orm.reconstructor
    def reconstruct(self):
        self.title = self.name
        self.subtitle = self.description
        self.tag = "mdi-share-variant"
        if self.enabled is False or not self.event_id:
            self.status = "red"
        else:
            self.status = "green"

    @classmethod
    def find(cls, node_id):
        return cls.query.get(node_id)

    @classmethod
    def get(cls, search):
        query = cls.query

        if search is not None:
            search_string = "%" + search.lower() + "%"
            query = query.filter(
                or_(
                    func.lower(RemoteNode.name).like(search_string),
                    func.lower(RemoteNode.description).like(search_string),
                )
            )

        return query.order_by(db.asc(RemoteNode.name)).all(), query.count()

    @classmethod
    def get_all_json(cls, search):
        remote_nodes, count = cls.get(search)
        schema = RemoteNodePresentationSchema(many=True)
        return {"total_count": count, "items": schema.dump(remote_nodes)}

    @classmethod
    def add(cls, data):
        schema = NewRemoteNodeSchema()
        remote_node = schema.load(data)
        db.session.add(remote_node)
        db.session.commit()

    @classmethod
    def delete(cls, remote_node_id):
        remote_node = cls.query.get(remote_node_id)
        db.session.delete(remote_node)
        db.session.commit()

    @classmethod
    def update(cls, remote_node_id, data):
        schema = NewRemoteNodeSchema()
        updated_remote_node = schema.load(data)
        remote_node = cls.query.get(remote_node_id)
        remote_node.name = updated_remote_node.name
        remote_node.description = updated_remote_node.description
        remote_node.enabled = updated_remote_node.enabled
        remote_node.remote_url = updated_remote_node.remote_url
        remote_node.events_url = updated_remote_node.events_url
        remote_node.access_key = updated_remote_node.access_key
        remote_node.sync_news_items = updated_remote_node.sync_news_items
        remote_node.sync_report_items = updated_remote_node.sync_report_items
        remote_node.osint_source_group_id = updated_remote_node.osint_source_group_id
        if remote_node.enabled is False:
            remote_node.event_id = None
        db.session.commit()
        return remote_node.enabled

    def connect(self, access_info):
        self.event_id = access_info["event_id"]
        self.last_synced_news_items = datetime.strptime(access_info["last_synced_news_items"], "%Y-%m-%d %H:%M:%S.%f")
        self.last_synced_report_items = datetime.strptime(access_info["last_synced_report_items"], "%Y-%m-%d %H:%M:%S.%f")
        db.session.commit()

    def disconnect(self):
        self.event_id = None
        db.session.commit()



from flask_sqlalchemy import SQLAlchemy
from flask_migrate import Migrate
from core.managers.db_seed_manager import pre_seed


db = SQLAlchemy()
migrate = Migrate()


def initialize(app):
    db.init_app(app)
    migrate.init_app(app, db)
    db.create_all()
    pre_seed(app)



__all__ = [
    "user",
    "role",
    "permission",
    "news_item",
    "collectors_node",
    "collector",
    "osint_source",
    "parameter",
    "parameter_value",
    "report_item",
    "attribute",
    "report_item_type",
    "address",
    "organization",
    "acl_entry",
    "product_type",
    "product",
    "presenters_node",
    "presenter",
    "publishers_node",
    "publisher_preset",
    "publisher",
    "word_list",
    "asset",
    "bot",
    "bots_node",
    "bot_preset",
    "notification_template",
    "remote",
    "tag_cloud",
    "token_blacklist",
]



from marshmallow import post_load, fields
from sqlalchemy import func, or_, orm

from core.managers.db_manager import db
from core.model.address import NewAddressSchema
from shared.schema.organization import OrganizationSchema, OrganizationPresentationSchema


class NewOrganizationSchema(OrganizationSchema):
    address = fields.Nested(NewAddressSchema)

    @post_load
    def make(self, data, **kwargs):
        return Organization(**data)


class Organization(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(), nullable=False)
    description = db.Column(db.String())

    address_id = db.Column(db.Integer, db.ForeignKey("address.id"))
    address = db.relationship("Address", cascade="all")

    def __init__(self, id, name, description, address):
        self.id = None
        self.name = name
        self.description = description
        self.address = address
        self.title = ""
        self.subtitle = ""
        self.tag = ""

    @orm.reconstructor
    def reconstruct(self):
        self.title = self.name
        self.subtitle = self.description
        self.tag = "mdi-office-building"

    @classmethod
    def find(cls, organization_id):
        return cls.query.get(organization_id)

    @classmethod
    def get_all(cls):
        return cls.query.order_by(db.asc(Organization.name)).all()

    @classmethod
    def get(cls, search):
        query = cls.query

        if search is not None:
            search_string = f"%{search.lower()}%"
            query = query.filter(
                or_(
                    func.lower(Organization.name).like(search_string),
                    func.lower(Organization.description).like(search_string),
                )
            )

        return query.order_by(db.asc(Organization.name)).all(), query.count()

    @classmethod
    def get_all_json(cls, search):
        organizations, count = cls.get(search)
        organizations_schema = OrganizationPresentationSchema(many=True)
        return {"total_count": count, "items": organizations_schema.dump(organizations)}

    @classmethod
    def add_new(cls, data):
        new_organization_schema = NewOrganizationSchema()
        organization = new_organization_schema.load(data)
        db.session.add(organization.address)
        db.session.add(organization)
        db.session.commit()

    @classmethod
    def update(cls, organization_id, data):
        schema = NewOrganizationSchema()
        updated_organization = schema.load(data)
        organization = cls.query.get(organization_id)
        organization.name = updated_organization.name
        organization.description = updated_organization.description
        organization.address.street = updated_organization.address.street
        organization.address.city = updated_organization.address.city
        organization.address.zip = updated_organization.address.zip
        organization.address.country = updated_organization.address.country
        db.session.commit()

    @classmethod
    def delete(cls, id):
        organization = cls.query.get(id)
        db.session.delete(organization)
        db.session.commit()



from core.managers import time_manager
from core.model.tag_cloud import TagCloud


def job(app):

    with app.app_context():
        TagCloud.delete_words()


def initialize(app):

    time_manager.schedule_job_every_day("00:01", job, app)



from pydantic import BaseSettings, validator
from typing import Optional


class Settings(BaseSettings):
    class Config:
        env_file = ".env"
        env_file_encoding = "utf-8"

    API_KEY: str
    MODULE_ID: str = "Core"
    DEBUG: bool = False
    SECRET_KEY: str

    JWT_IDENTITY_CLAIM: str = "sub"
    JWT_ACCESS_TOKEN_EXPIRES: int = 14400
    REDIS_URL: str = "redis://localhost"

    DB_URL: str = "localhost"
    DB_DATABASE: str = "taranis"
    DB_USER: str = "taranis"
    DB_PASSWORD: str = "supersecret"
    SQLALCHEMY_SCHEMA: str = "postgresql+psycopg2"
    SQLALCHEMY_ECHO: bool = False
    SQLALCHEMY_TRACK_MODIFICATIONS: bool = False
    SQLALCHEMY_DATABASE_URI: Optional[str]
    COLORED_LOGS: bool = True
    OpenAPI: str = "static/"

    @validator("SQLALCHEMY_DATABASE_URI", pre=True, always=True)
    def set_sqlalchemy_uri(cls, value, values):
        if value and len(value) > 1:
            return value
        return f"{values['SQLALCHEMY_SCHEMA']}://{values['DB_USER']}:{values['DB_PASSWORD']}@{values['DB_URL']}/{values['DB_DATABASE']}"

    # if "postgresql" in SQLALCHEMY_DATABASE_URI:
    #     DB_POOL_SIZE: int = 10
    #     DB_POOL_RECYCLE: int = 300
    #     DB_POOL_TIMEOUT: int = 5

    #     SQLALCHEMY_ENGINE_OPTIONS: dict = {
    #         "pool_size": DB_POOL_SIZE,
    #         "pool_recycle": DB_POOL_RECYCLE,
    #         "pool_pre_ping": True,
    #         "pool_timeout": DB_POOL_TIMEOUT,
    #     }

    OPENID_CLIENT_ID: Optional[str]
    OPENID_CLIENT_SECRET: Optional[str]
    OPENID_METADAT_URL: str = "http://keycloak/realms/master/.well-known/openid-configuration"


Config = Settings()



from flask import request
from flask_restful import Resource, reqparse

from core.managers import sse_manager, collectors_manager
from core.managers.auth_manager import api_key_required
from core.managers.log_manager import logger
from core.model import osint_source, collectors_node, news_item
from shared.schema.osint_source import OSINTSourceUpdateStatusSchema


class OSINTSourcesForCollectors(Resource):
    @api_key_required
    def get(self, collector_id):
        try:
            parser = reqparse.RequestParser()
            parser.add_argument("collector_type", location="args", required=True)
            parameters = parser.parse_args()

            logger.log_debug(f"Searching node: {parameters}")
            node = collectors_node.CollectorsNode.get_by_id(collector_id)
            logger.log_debug(f"Node found: {node}")
            if not node:
                return "", 404

            node.updateLastSeen()
        except Exception:
            logger.log_debug_trace()

        return osint_source.OSINTSource.get_all_for_collector_json(node, parameters.collector_type)


class AddNewsItems(Resource):
    @api_key_required
    def post(self):
        json_data = request.json
        osint_source_ids = news_item.NewsItemAggregate.add_news_items(json_data)
        sse_manager.news_items_updated()
        sse_manager.remote_access_news_items_updated(osint_source_ids)


class CollectorsNode(Resource):
    @api_key_required
    def get(self, node_id):
        return collectors_node.CollectorsNode.get_json_by_id(node_id)

    @api_key_required
    def put(self, node_id):
        return collectors_manager.update_collectors_node(node_id, request.json)

    @api_key_required
    def post(self):
        return collectors_manager.add_collectors_node(request.json)

    @api_key_required
    def delete(self, node_id):
        collectors_node.CollectorsNode.delete(node_id)


class OSINTSourceStatusUpdate(Resource):
    @api_key_required
    def put(self, osint_source_id):
        source = osint_source.OSINTSource.get_by_id(osint_source_id)
        if not source:
            return {}, 404

        try:
            osint_source_status_schema = OSINTSourceUpdateStatusSchema()
            osint_source_status = osint_source_status_schema.load(request.json)
            logger.debug(f"osint_source_status: ${osint_source_status}")
        except Exception:
            logger.log_debug_trace()
            return {}, 400

        return {}, 200


class CollectorStatusUpdate(Resource):
    @api_key_required
    def get(self, node_id):
        collector = collectors_node.CollectorsNode.get_by_id(node_id)
        if not collector:
            return "", 404

        try:
            collector.updateLastSeen()
        except Exception as ex:
            logger.log_debug(ex)
            return {}, 400

        return {}, 200


def initialize(api):
    api.add_resource(
        OSINTSourcesForCollectors,
        "/api/v1/collectors/<string:collector_id>/osint-sources",
    )
    api.add_resource(
        OSINTSourceStatusUpdate,
        "/api/v1/collectors/osint-sources/<string:osint_source_id>",
    )
    api.add_resource(CollectorStatusUpdate, "/api/v1/collectors/<string:node_id>")
    api.add_resource(AddNewsItems, "/api/v1/collectors/news-items")
    api.add_resource(CollectorsNode, "/api/v1/collectors/node/<string:node_id>", "/api/v1/collectors/node")



import requests


class BotsApi:
    def __init__(self, api_url, api_key):
        self.api_url = api_url
        if self.api_url.endswith("/"):
            self.api_url = self.api_url[:-1]
        self.api_key = api_key
        self.headers = {"Authorization": f"Bearer {self.api_key}"}

    def get_bots_info(self):
        response = requests.get(f"{self.api_url}/api/v1/bots", headers=self.headers)
        return response.json(), response.status_code
